from policyengine_canada.model_api import *


class yt_child_benefit(Variable):
    value_type = float
    entity = Household
    label = "Yukon child benefit"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.yt.benefits.child_benefit
        income = household("adjusted_family_net_income", period)
        children = household("yt_child_benefit_eligible_children", period)
        base = household("yt_child_benefit_base", period)
        reduction = select(
            [children == 1, children > 1],
            [
                p.one_child.reduction_rate.calc(income),
                p.multiple_children.reduction_rate.calc(income),
            ],
            default=0,
        )
        return max_(0, base - reduction)



from policyengine_canada.model_api import *


class bc_income_tax_before_credits(Variable):
    value_type = float
    entity = Person
    label = "British Columbia income tax"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/frequently-asked-questions-individuals/canadian-income-tax-rates-individuals-current-previous-years.html"
    defined_for = ProvinceCode.BC

    def formula(person, period, parameters):
        income = person("bc_taxable_income", period)
        p = parameters(period).gov.provinces.bc.tax.income.rate
        return p.calc(income)



from policyengine_canada.model_api import *


class yt_cftc_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Yukon Normal Fitness Fee eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(
            period
        ).gov.provinces.yt.tax.income.credits.childrens_fitness
        disabled = person("is_disabled", period) & (
            age < p.disability_supplement.age
        )
        return disabled | (age < p.child_age_eligibility)



from policyengine_canada.model_api import *


class tuition_expenses(Variable):
    value_type = float
    entity = Person
    label = "Tuition Expenses"
    unit = CAD
    documentation = "Tuition expenses & Fees incurred"
    definition_period = YEAR



from policyengine_canada.model_api import *


class sk_active_family_benefit(Variable):
    value_type = float
    entity = Household
    label = "Saskatchewan Active Family Benefit"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(period).gov.provinces.sk.benefits.afb
        person = household.members
        child = person("age", period) <= p.eligibility.age
        disabled = person("is_disabled", period)
        children = household.sum(child)
        disabled_children = household.sum(child & disabled)
        non_disabled_children = children - disabled_children
        eligible = income <= p.eligibility.income
        amount_if_eligible = (
            p.amount.non_disabled * non_disabled_children
        ) + (p.amount.disabled * disabled_children)
        return eligible * amount_if_eligible



from policyengine_canada.model_api import *


class ProvinceName(Enum):
    AB = "Alberta"
    BC = "British Columbia"
    MB = "Manitoba"
    NB = "New Brunswick"
    NL = "Newfoundland and Labrador"
    NS = "Nova Scotia"
    NT = "Northwest Territories"
    NU = "Nunavut"
    ONT = "Ontario"
    PE = "Prince Edward Island"
    QC = "Quebec"
    SK = "Saskatchewan"
    YT = "Yukon"


class province_name(Variable):
    value_type = Enum
    possible_values = ProvinceName
    default_value = ProvinceName.ONT
    entity = Household
    label = "Province"
    definition_period = YEAR



from policyengine_canada.model_api import *


class gst_credit_singles_boost(Variable):
    value_type = float
    entity = Household
    label = "GST Credit Additional Amount for Singles"
    unit = CAD
    documentation = "Single-adult households without children get 2 percent of the difference between family net income and the threshold amount, up to a maximum of $161. Single-adult households _with_ children always get the maximum of $161, regardless of household net income."
    definition_period = YEAR

    # Singles get an amount between 0 and 161, but single _parents_ always get the full 161.
    def formula(household, period, parameters):
        single_parent_household = household(
            "gst_credit_single_parent_household", period
        )
        net_income = household("household_net_income", period)
        p = parameters(
            period
        ).gov.cra.tax.income.credits.gst_credit.singles_boost
        singles_phase_in = p.phase_in.calc(net_income)
        singles_amount = min_(p.cap, singles_phase_in)
        amount_if_single = where(
            single_parent_household, p.cap, singles_amount
        )
        eligible = ~household("is_married", period)
        return eligible * amount_if_single



"""
This file defines the entities needed by our legislation.

Taxes and benefits can be calculated for different entities: persons, household, companies, etc.

See https://openfisca.org/doc/key-concepts/person,_entities,_role.html
"""

from policyengine_core.entities import build_entity

Household = build_entity(
    key="household",
    plural="households",
    label="Household",
    doc="A group of people sharing a living space",
    roles=[
        {
            "key": "member",
            "plural": "members",
            "label": "Member",
            "doc": "A member of the household.",
        },
    ],
)

Person = build_entity(
    key="person",
    plural="people",
    label="Person",
    doc="A single person",
    is_person=True,
)


entities = [Household, Person]



from policyengine_canada.model_api import *


class sk_low_income_credit(Variable):
    value_type = float
    entity = Household
    label = "Sasktachewan low income tax credit post reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.sk.tax.income.credits.slic
        base = household("sk_low_income_credit_base", period)
        income = household("adjusted_family_net_income", period)
        return max_(0, base - p.phase_out.calc(income))



from policyengine_canada.model_api import *


class dtc_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Is eligible for the disability tax credit"
    definition_period = YEAR



from policyengine_canada.model_api import *


class home_energy_costs_on_a_reserve(Variable):
    value_type = float
    entity = Household
    label = "Amounts paid for energy on a reserve"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class maternity_and_parental_benefit(Variable):
    value_type = float
    entity = Person
    label = "Employment Insurance maternity and parental benefits"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


# SPSD/M 29.0: imioas or imoasmax, I'm not sure. I think imioas is post-repayment?
class oas_net(Variable):
    value_type = float
    entity = Person
    label = "Old age security pension pre-repayment"
    documentation = "The OAS amount a person is eligible for prior to the repayment tax. See SPSD/M 'imoasmax'."
    unit = CAD
    definition_period = YEAR
    adds = ["oas_pre_repayment"]
    subtracts = ["oas_repayment"]



from policyengine_canada.model_api import *


class on_child_benefit_reduction(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Benefit reduction"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        return parameters(period).gov.provinces.on.benefits.ocb.reduction.calc(
            income
        )



from policyengine_canada.model_api import *


class ns_low_income_tax_reduction_base_children(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia low income tax reduction base for children"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        p = parameters(
            period
        ).gov.provinces.ns.tax.income.credits.low_income_tax_reduction
        children = household(
            "ns_low_income_tax_reduction_eligible_children", period
        )
        return children * p.base.dependant.base



from policyengine_canada.model_api import *


class is_child_for_gst_credit(Variable):
    value_type = bool
    entity = Person
    label = "Is the child below the age limit for a household to get a credit for this child?"
    definition_period = YEAR

    def formula(person, period, parameters):
        dependant = person("is_dependant", period)
        age = person("age", period)
        adult_age = parameters(
            period
        ).gov.cra.tax.income.credits.gst_credit.adult_age
        return dependant & (age < adult_age)



from policyengine_canada.model_api import *


class cwb_eligible(Variable):
    value_type = bool
    entity = Household
    label = "Eligible for Canada Workers Benefit"
    definition_period = YEAR

    def formula(household, period, parameters):
        person = household.members
        p = parameters(period).gov.cra.benefits.cwb
        eligible_age = person("age", period) >= p.eligible_age
        family = household("is_cwb_family", period)
        any_of_eligible_age = household.any(eligible_age)
        return family | any_of_eligible_age



from policyengine_canada.model_api import *


class childcare_received_hours_per_day(Variable):
    value_type = int
    entity = Household
    label = "Child care hours per day"
    unit = "hour"
    definition_period = YEAR



from policyengine_canada.model_api import *


class other_employment_income(Variable):
    value_type = float
    entity = Person
    label = "Other employment income"
    unit = CAD
    documentation = "Employment income reported on Line 10400 of the personal income tax return"
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/topics/about-your-tax-return/tax-return/completing-a-tax-return/personal-income/line-10400-other-employment-income.html"



from policyengine_canada.model_api import *


class oshptg_adjusted_oeptc(Variable):
    value_type = float
    entity = Household
    documentation = "Ontario energy and property tax credit adjusted for the Ontario senior homeowners property tax grant"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        person = household.members
        oshptg = person("on_senior_homeowners_property_tax_grant", period)
        eligible = oshptg > 0
        oeptc = household("oeptc", period)
        adjusted_oeptc = oshptg + oeptc
        energy_component = household("oeptc_energy_component", period)
        adjusted_for_energy_component = adjusted_oeptc - energy_component
        occupany_costs = household("oeptc_occupancy_cost", period)
        adjusted_for_occupany_costs = (
            adjusted_for_energy_component - occupany_costs
        )
        return max_(eligible * (oeptc - adjusted_for_occupany_costs), 0)



from policyengine_canada.model_api import *


class nb_benefits(Variable):
    value_type = float
    entity = Household
    label = "New Brunswick benefits"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.NB
    adds = "gov.provinces.nb.benefits.benefits"



from policyengine_canada.model_api import *


class oas_eligibility(Variable):
    value_type = bool
    entity = Person
    label = "Old age security pension eligibility"
    unit = CAD
    documentation = "Eligibility for the OAS based on age and adult years resident in canada"
    definition_period = YEAR

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.cra.benefits.old_age_security_pension.eligibility
        age_eligible = person("age", period) >= p.age.base
        residency_eligible = (
            person("adult_years_in_canada", period) >= p.residence.any
        )
        return age_eligible & residency_eligible



from policyengine_canada.model_api import *


class climate_action_incentive(Variable):
    value_type = float
    entity = Household
    label = "Climate action incentive payment"
    unit = CAD
    documentation = "Universal amount without adjustment based on AFNI"
    definition_period = YEAR

    def formula(household, period, parameters):
        amount = household("climate_action_incentive_pre_rural", period)
        rural = household("is_rural", period)
        rural_percent_bonus = parameters(
            period
        ).gov.cra.tax.income.credits.climate_action_incentive.rural
        return amount * (1 + rural * rural_percent_bonus)



from policyengine_canada.model_api import *


class nb_low_income_tax_reduction_base(Variable):
    value_type = float
    entity = Household
    label = "New Brunswick low income tax reduction base"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    def formula(household, period, parameters):
        person = household.members
        p = parameters(
            period
        ).gov.provinces.nb.tax.income.credits.low_income_tax_reduction
        married = household("is_married", period)
        dependant = person("is_dependant", period)
        return min_(
            p.base.max_amount,
            p.base.head
            + married * p.base.spouse
            + dependant * p.base.dependant,
        )



from policyengine_canada.model_api import *


class qc_fa_eligibility(Variable):
    value_type = bool
    entity = Household
    label = "Quebec family allowance eligible household"
    reference = "https://www.legisquebec.gouv.qc.ca/en/document/cs/I-3?langCont=en#se:1029_8_61_8"
    definition_period = YEAR
    defined_for = ProvinceCode.QC
    adds = ["own_children_in_household"]

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.fa

        person = household.members
        # is parent and resides with of the eligible dependent child
        is_parent = person("own_children_in_household", period) > 0

        return household.any(is_parent)



from policyengine_canada.model_api import *


class oeptc(Variable):
    value_type = float
    entity = Household
    label = "Ontario energy and property tax credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(household, period, parameters):
        energy_plus_property_tax_components = add(
            household,
            period,
            ["oeptc_energy_component", "oeptc_property_tax_component"],
        )
        afni = household("adjusted_family_net_income", period)
        senior_status = household("oeptc_senior_status", period)
        category = household("oeptc_category", period)
        p = parameters(
            period
        ).gov.provinces.on.tax.income.credits.oeptc.phase_out
        excess = max_(afni - p.start[senior_status][category], 0)
        phase_out = excess * p.rate
        phased_out = energy_plus_property_tax_components - phase_out
        shared_custody_phased_out_divided = (
            phased_out / p.shared_custody.divisor
        )
        shared_custody_excess = max_(
            afni - p.shared_custody.start[senior_status], 0
        )
        shared_custody_second_phase_out = shared_custody_excess * p.rate
        shared_custody_second_phased_out = (
            energy_plus_property_tax_components
            - shared_custody_second_phase_out
        )
        shared_custody_second_phased_out_divided = (
            shared_custody_second_phased_out / p.shared_custody.divisor
        )
        shared_custody_result = (
            shared_custody_phased_out_divided
            + shared_custody_second_phased_out_divided
        )
        return max_(
            0,
            where(
                category == category.possible_values.SINGLE_SHARED_CUSTODY,
                shared_custody_result,
                phased_out,
            ),
        )



from policyengine_canada.model_api import *


class nb_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = Person
    label = "New Brunswick income tax before refundable credits"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/frequently-asked-questions-individuals/canadian-income-tax-rates-individuals-current-previous-years.html"
    adds = ["nb_income_tax_before_credits"]
    defined_for = ProvinceCode.NB



from policyengine_canada.model_api import *


class childcare_expense(Variable):
    value_type = float
    entity = Person
    label = "Expense for child care"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class on_sales_tax_credit_base(Variable):
    value_type = float
    entity = Household
    label = "Ontario Sales Tax Credit Base"
    unit = CAD
    documentation = (
        "Base amount of Ontario sales tax credits before reduction."
    )
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(household, period, parameters):
        members = household("household_size", period)
        p = parameters(period).gov.provinces.on.tax.income.credits.ostc
        return members * p.amount



from policyengine_canada.model_api import *


class acfb_working_component_base(Variable):
    value_type = float
    entity = Household
    label = "Alberta child and family benefit working component phased-in base amount"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.ab.benefits.acfb.working_component
        employment_income = household("family_employment_income", period)
        eligible_children = household("acfb_eligible_children", period)
        max_amount = (
            (p.base.one_child * (eligible_children > 0))
            + (p.base.two_children * (eligible_children > 1))
            + (p.base.three_children * (eligible_children > 2))
            + (p.base.four_or_more_children * (eligible_children > 3))
        )
        eligible = employment_income > p.phase_in.start
        phase_in = (employment_income - p.phase_in.start) * p.phase_in.rate
        phased_in = min_(max_amount, phase_in)
        return eligible * phased_in



from policyengine_canada.model_api import *


class ns_poverty_reduction_credit(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia Poverty Reduction Credit"
    unit = CAD
    definition_period = YEAR
    reference = (
        "https://novascotia.ca/coms/noteworthy/PovertyReductionCredit.html",
        "https://nslegislature.ca/sites/default/files/legc/statutes/income%20tax.pdf#page=121",
    )
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        children = household("count_children", period)
        income_assistance = household("ns_income_assistance", period)
        p = parameters(period).gov.provinces.ns.tax.income.credits.prc
        eligible = (
            (children == 0)
            & (income < p.income_threshold)
            & (income_assistance > 0)
        )
        return where(eligible, p.amount, 0)



from policyengine_canada.model_api import *


class climate_action_incentive_person(Variable):
    value_type = float
    entity = Person
    label = "Canada Climate Action amount per individual"
    unit = CAD
    documentation = "Determination of the amount per individual"
    definition_period = YEAR

    def formula(person, period, parameters):
        province = person.household("province_code_str", period)
        category = person("climate_action_incentive_category", period)
        amounts = parameters(
            period
        ).gov.cra.tax.income.credits.climate_action_incentive.amount
        return amounts[category][province]



from policyengine_canada.model_api import *


class deductions_from_total_to_net_income(Variable):
    value_type = float
    entity = Person
    label = "Deductions from Total to Net Income"
    unit = CAD
    documentation = "Deductions used in the calculation of Net Income"
    definition_period = YEAR
    reference = "SPSD/M 29.0 Variable Guide, Page 8"



from policyengine_canada.model_api import *


def create_in_province_variable(province: str) -> Type[Variable]:
    return type(
        province,
        (Variable,),
        {
            "definition_period": YEAR,
            "label": f"In {province}",
            "value_type": bool,
            "entity": Household,
            "formula": in_province(province),
            "module_name": "",
            "hidden_input": True,
        },
    )


def create_10_province_variables() -> List[Type[Variable]]:
    return [create_in_province_variable(province) for province in PROVINCES]



from policyengine_canada.model_api import *


class acfb_base_component_base(Variable):
    value_type = float
    entity = Household
    label = "Alberta child and family benefit base component base amount"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(household, period, parameters):
        p = parameters(
            period
        ).gov.provinces.ab.benefits.acfb.base_component.base
        eligible_children = household("acfb_eligible_children", period)
        return (
            (p.one_child * (eligible_children > 0))
            + (p.two_children * (eligible_children > 1))
            + (p.three_children * (eligible_children > 2))
            + (p.four_or_more_children * (eligible_children > 3))
        )



from policyengine_canada.model_api import *


class nb_child_benefit(Variable):
    value_type = int
    entity = Household
    label = "New Brunswick child tax benefit"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        children = household("nb_child_benefit_eligible_children", period)
        p = parameters(period).gov.provinces.nb.benefits.nbcb
        base = p.base * children
        supplement = household("nb_child_benefit_supplement", period)
        one_child_amount = max_(base - p.phase_out.one_child.calc(income), 0)
        multiple_children_amount = max_(
            base - p.phase_out.multiple_children.calc(income),
            0,
        )
        pre_supplement_amount = select(
            # Conditions.
            [children == 1, children > 1],
            # Results.
            [
                one_child_amount,
                multiple_children_amount,
            ],
            default=0,
        )
        return pre_supplement_amount + supplement



from policyengine_canada.model_api import *


class ns_low_income_tax_reduction_eligible_children(Variable):
    value_type = int
    entity = Household
    label = "Nova Scotia low income tax reduction eligible children"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    adds = ["ns_low_income_tax_reduction_eligible_child"]



from policyengine_canada.model_api import *


class canadian_armed_forces_and_personnel_deduction(Variable):
    value_type = float
    entity = Person
    label = "Canadian armed forces and personnel deduction"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class child_disability_benefit_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Child Disability Benefit"
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/child-family-benefits/child-disability-benefit.html"

    def formula(person, period, parameters):
        dtc_eligible = person("dtc_eligible", period)
        ccb_eligible = person("child_benefit_eligible", period)
        return ccb_eligible & dtc_eligible



from policyengine_canada.model_api import *


class cwb_base_max_amount(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit max base amount"
    definition_period = YEAR
    defined_for = "cwb_eligible"
    unit = CAD

    def formula(household, period, parameters):
        p = parameters(period).gov.cra.benefits.cwb.amount
        family = household("is_cwb_family", period)
        return where(family, p.family, p.single)



from policyengine_canada.model_api import *


class qc_sa_married_one_eligible(Variable):
    value_type = int
    entity = Household
    label = "Quebec senior assistance tax credits for family with only one eligible senior"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.sa

        person = household.members

        age_eligible = person("age", period) >= p.age_eligibility
        spouse_eligible = person("qc_sa_spouse_eligible", period)

        # ^ is XOR (Exclusive OR)
        # only one senior in the household is eligible (either the spouse or the head of the household, not both)
        eligible = age_eligible ^ spouse_eligible
        return household.sum(eligible)



from policyengine_canada.model_api import *


class bc_climate_action_tax_credit_dependant_children(Variable):
    value_type = int
    entity = Household
    label = "British Columbia Climate action tax credit dependant Children"
    unit = CAD
    documentation = "Number of eligible dependant children"
    definition_period = YEAR
    defined_for = ProvinceCode.BC
    adds = ["is_child_for_bc_climate_action_tax_credit"]



from policyengine_canada.model_api import *


class is_married(Variable):
    value_type = bool
    entity = Household
    label = "Is married"
    documentation = "Household head is married"
    definition_period = YEAR

    def formula(household, period, parameters):
        return add(household, period, ["is_spouse"]) > 0



from policyengine_canada.model_api import *


class universal_child_care_benefit(Variable):
    value_type = float
    entity = Person
    label = "Universal child care benefit"
    unit = CAD
    documentation = "Universal child care benefit (UCCB) ended in 2016."
    definition_period = YEAR



from policyengine_canada.model_api import *


class on_child_benefit(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Benefit"
    unit = CAD
    documentation = (
        "Non taxable amount paid monthly per children under 18 years of age. "
    )
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(household, period, parameters):
        base = household("on_child_benefit_base", period)
        reduction = household("on_child_benefit_reduction", period)
        shared_custody_divisor = parameters(
            period
        ).gov.provinces.on.benefits.ocb.shared_custody_divisor
        has_any_full_custody_children = (
            add(household, period, ["full_custody"]) > 0
        )
        divisor = where(
            has_any_full_custody_children, 1, shared_custody_divisor
        )
        child_benefit_amount = max_(0, base - reduction)
        return child_benefit_amount / divisor



from policyengine_canada.model_api import *


class bc_age_credit(Variable):
    value_type = float
    entity = Person
    label = "British Columbia Age credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(period).gov.provinces.bc.tax.income.credits.age
        eligible = age >= p.eligible_age
        income = person("total_individual_pre_tax_income", period)
        reduction = p.phase_out.rate.calc(income)
        return eligible * (max_(0, p.base - reduction))



from policyengine_canada.model_api import *


class on_senior_homeowners_property_tax_grant_reduction(Variable):
    value_type = float
    entity = Person
    label = "Ontario senior homeowners property tax grant reduction"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(person, period, parameters):
        income = person.household("adjusted_family_net_income", period)
        married = person.household("is_married", period)
        p = parameters(period).gov.provinces.on.tax.grants.oshptg.reduction
        return where(married, p.family.calc(income), p.single.calc(income))



from policyengine_canada.model_api import *


class gst_credit_base(Variable):
    value_type = float
    entity = Household
    label = "GST Credit before the boost is applied and before reduction based on family net income"
    unit = CAD
    definition_period = YEAR

    adds = ["gst_credit_person"]



from policyengine_canada.model_api import *


class person_index(Variable):
    value_type = int
    entity = Person
    label = "Person reference number, based on age"
    definition_period = YEAR

    def formula(person, period, parameters):
        # The person index, by age, descending.
        return person.get_rank(person.household, -person("age", period))



from policyengine_canada.model_api import *


class training_credit(Variable):
    value_type = float
    entity = Person
    label = "Training credit"
    unit = CAD
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        income = tax_unit("training_credit_income", period)
        tuition = tax_unit("tuition_expenses", period)
        age = tax_unit("age", period)
        training = parameters(period).gov.cra.tax.income.credits.training
        lower_limit = training.age_eligibility.min
        upper_limit = training.age_eligibility.max
        meets_age_requirement = (age >= lower_limit) & (age <= upper_limit)
        existing_credits = tax_unit("prior_training_credits", period)
        cap = training.lifetime_cap
        remaining = max_(0, cap - existing_credits)
        threshold = training.amount
        credits = threshold.calc(income)
        student = tuition > 0
        amount_if_eligible = min_(remaining, credits)
        eligible = meets_age_requirement & student
        return eligible * amount_if_eligible



from policyengine_canada.model_api import *


class income_tax_before_credits(Variable):
    value_type = float
    entity = Person
    label = "Income tax before non-refundable tax credits"
    unit = CAD
    documentation = "Example income tax regime"
    definition_period = YEAR

    def formula(person, period, parameters):
        income = person("total_individual_pre_tax_income", period)
        gov = parameters(period).gov.cra.tax.income
        return gov.income_tax_schedule.calc(income)



from policyengine_canada.model_api import *


class ns_low_income_tax_reduction_reduction(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia low income tax reduction reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        return parameters(
            period
        ).gov.provinces.ns.tax.income.credits.low_income_tax_reduction.phase_out.calc(
            income
        )



from policyengine_canada.model_api import *


class child_disability_benefit_children(Variable):
    value_type = int
    entity = Household
    label = "Children eligible for the Child Disability Benefit"
    definition_period = YEAR

    adds = ["child_disability_benefit_eligible"]



from policyengine_canada.model_api import *

label = "Earnings"


class family_employment_income(Variable):
    value_type = float
    entity = Household
    label = "Family Employment income"
    unit = CAD
    documentation = "Income of families from gainful employment"
    definition_period = YEAR

    adds = ["employment_income"]



from policyengine_canada.model_api import *


class is_child_of_filer(Variable):
    value_type = bool
    entity = Person
    label = "Is the child of the tax filer or their spouse"
    definition_period = YEAR



from policyengine_canada.model_api import *


class qc_sa_single(Variable):
    value_type = bool
    entity = Household
    label = "Quebec senior assistance tax credits eligible senior did not have a spouse"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        person = household.members

        p = parameters(period).gov.provinces.qc.tax.income.credits.sa

        age_eligible = person("age", period) >= p.age_eligibility

        return age_eligible & ~person("is_spouse", period)



from policyengine_canada.model_api import *


class ns_low_income_tax_reduction_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Nova Scotia low income tax reduction eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(person, period, parameters):
        age = person("age", period)
        return (
            age
            <= parameters(
                period
            ).gov.provinces.ns.tax.income.credits.low_income_tax_reduction.base.dependant.age_eligibility
        )



from policyengine_canada.model_api import *


class oeptc_child(Variable):
    value_type = bool
    entity = Person
    label = "Is a child for the Ontario Energy and Property Tax Credit"
    definition_period = YEAR

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(period).gov.provinces.on.tax.income.credits.oeptc
        return age < p.child_max_age



from policyengine_canada.model_api import *


class acfb_base_component(Variable):
    value_type = float
    entity = Household
    label = "Alberta child and family benefit base component reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(household, period, parameters):
        reduction = household("acfb_base_component_reduction", period)
        base = household("acfb_base_component_base", period)
        return max_(0, base - reduction)



from policyengine_canada.model_api import *


class child_benefit_reduction(Variable):
    value_type = float
    entity = Household
    label = "Canada Child Benefit reduction"
    unit = CAD
    documentation = (
        "Non taxable amount paid monthly per children under 18 years of age. "
    )
    definition_period = YEAR

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        children = household("child_benefit_eligible_children", period)
        p = parameters(period).gov.cra.benefits.ccb.reduction
        return select(
            # Conditions.
            [children == 1, children == 2, children == 3, children > 3],
            # Results.
            [
                p.one_child.calc(income),
                p.two_children.calc(income),
                p.three_children.calc(income),
                p.four_or_more_children.calc(income),
            ],
            default=0,
        )



from policyengine_canada.model_api import *


class dtc_base(Variable):
    value_type = float
    entity = Person
    label = "Disability tax credit base"
    unit = CAD
    definition_period = YEAR
    defined_for = "dtc_eligible"

    def formula(person, period, parameters):
        return parameters(period).gov.cra.tax.income.credits.dtc.base



from policyengine_canada.model_api import *


class is_emancipated(Variable):
    value_type = bool
    entity = Person
    label = "Is emancipated by a competent authority"
    definition_period = YEAR



from policyengine_canada.model_api import *


class on_benefits(Variable):
    value_type = float
    entity = Household
    label = "Ontario benefits"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT
    adds = "gov.provinces.on.benefits.benefits"



from policyengine_canada.model_api import *


class canada_employment_amount(Variable):
    value_type = float
    entity = Person
    label = "Canada Employment Amount"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/topics/about-your-tax-return/tax-return/completing-a-tax-return/deductions-credits-expenses/line-31260-canada-employment-amount.html"

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.cra.tax.income.credits.canada_employment_amount
        # While the government website states that the Canada Employment Amount is designed
        # to support workers with work-related expenses, those expenses are not part of the formula.
        countable_income = add(person, period, p.income_sources)
        return min_(countable_income, p.max_amount)



from policyengine_canada.model_api import *


class cwb_disability_supplement_phase_out(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit disability supplement phase out"
    definition_period = YEAR
    unit = CAD

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(
            period
        ).gov.cra.benefits.cwb.phase_out.disability_supplement
        category = household("cwb_disability_category", period)
        categories = category.possible_values
        return select(
            [
                category == categories.FAMILY_WITH_TWO_DISABLED_SPOUSES,
                category == categories.FAMILY_WITH_ONE_DISABLED_SPOUSE,
                category == categories.SINGLE,
            ],
            [
                p.family_with_two_spouses_disabled.calc(income),
                p.family_with_one_spouse_disabled.calc(income),
                p.single.calc(income),
            ],
            default=0,
        )



from policyengine_canada.model_api import *


class bc_benefits(Variable):
    value_type = float
    entity = Household
    label = "British Columbia benefits"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC
    adds = "gov.provinces.bc.benefits.benefits"



from policyengine_canada.model_api import *


class on_low_income_workers_tax_credit_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible person for the Ontario Low-Income Workers Tax Credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(person, period, parameters):
        return person("employment_income", period) > 0



from policyengine_canada.model_api import *


class in_need_of_protective_services(Variable):
    value_type = bool
    entity = Person
    label = "Person is in need of protective services"
    definition_period = YEAR
    reference = (
        "https://novascotia.ca/just/regulations/regs/esiaregs.htm#TOC3_8"
    )



from policyengine_canada.model_api import *


class self_employment_income(Variable):
    value_type = float
    entity = Person
    label = "Self-employment income"
    unit = CAD
    documentation = "Income from gainful employment"
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/topics/about-your-tax-return/tax-return/completing-a-tax-return/personal-income/self-employment-income-lines-13499-14299-gross-income-lines-13500-14300-net-income.html"



from policyengine_canada.model_api import *


class nb_child_benefit_supplement(Variable):
    value_type = int
    entity = Household
    label = "New Brunswick child benefit supplement"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(
            period
        ).gov.provinces.nb.benefits.nbcb.working_income_supplement
        working_income = household("family_working_income", period)
        # the supplement amount is reduced by 4% of "family earned income" in excess of $3,750 minus 5% of "family net income" in excess of $20,921.
        reduced_amount = max_(
            p.phase_in.calc(working_income) - p.phase_out.calc(income), 0
        )
        # The lesser of the base amount or teh reduced amount is utilized.
        return min_(p.base, reduced_amount)



from policyengine_canada.model_api import *


class yt_child_benefit_eligible_children(Variable):
    value_type = int
    entity = Household
    label = "Yukon child benefit eligible children"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    adds = ["yt_child_benefit_eligible_child"]



from policyengine_canada.model_api import *


class climate_action_incentive_pre_rural(Variable):
    value_type = float
    entity = Household
    label = "Canada climate action incentive before rural suplement"
    unit = CAD
    documentation = "Universal amount without adjustment based on AFNI"
    definition_period = YEAR

    adds = ["climate_action_incentive_person"]



from policyengine_canada.model_api import *


class dental_benefit_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for dental benefit"
    definition_period = YEAR

    def formula(person, period, parameters):
        p = parameters(period).gov.cra.benefits.dental_benefit
        # Determine eligibility.
        has_dental_expenses = person("dental_expenses", period) > 0
        has_private_insurance = person("has_private_dental_insurance", period)
        age_eligible = person("age", period) < p.ineligible_age
        # The amount does not vary with the dental costs.
        # Child only needs to have received some dental care.
        return ~has_private_insurance & has_dental_expenses & age_eligible



from policyengine_canada.model_api import *


class nu_cost_of_living_credit_supplement(Variable):
    value_type = float
    entity = Person
    label = "Nunavut cost of living credit supplement for single parents"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.provinces.nu.tax.income.credits.cost_of_living.supplement_for_single_parents
        adjusted_income = person(
            "nu_cost_of_living_credit_adjusted_net_income", period
        )
        excess = max_(0, adjusted_income - p.income)
        uncapped = excess * p.rate
        return min_(uncapped, p.max_amount)



from policyengine_canada.model_api import *


class resides_on_settlement_land(Variable):
    value_type = bool
    entity = Household
    label = "Resides on Settlement land"
    definition_period = YEAR



from policyengine_canada.model_api import *


class individual_net_income(Variable):
    value_type = float
    entity = Person
    label = "Individual Net Income"
    unit = CAD
    documentation = "The sum income from all sources, minus all eligible deductions such as RRSP contributions, childcare expenses, moving expenses, etc"
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/topics/about-your-tax-return/tax-return/completing-a-tax-return/deductions-credits-expenses/line-23600-net-income.html"

    def formula(tax_unit, period, parameters):
        total_income = tax_unit("total_individual_pre_tax_income", period)
        deductions_from_total_to_net_income = tax_unit(
            "deductions_from_total_to_net_income", period
        )
        return total_income - deductions_from_total_to_net_income



from policyengine_canada.model_api import *


class working_income(Variable):
    value_type = float
    entity = Person
    label = "Working income"
    unit = CAD
    documentation = "Income from employment and self-employment"
    definition_period = YEAR

    adds = "gov.cra.benefits.cwb.working_income_sources"



from policyengine_canada.model_api import *


class market_income(Variable):
    value_type = float
    entity = Household
    label = "Household market (pre-tax) income"
    unit = CAD
    definition_period = YEAR
    adds = ["total_individual_pre_tax_income"]



from policyengine_canada.model_api import *


class is_child_for_on_child_care_fee_subsidy_reduction(Variable):
    value_type = bool
    entity = Person
    label = "Is a child for the Ontario Child Care Fee Subsidy reduction"
    definition_period = YEAR

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(
            period
        ).gov.provinces.on.subsidies.on_child_care_fee_subsidy.reduction
        return age < p.age



from policyengine_canada.model_api import *


class nb_child_benefit_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "New Brunswick family benefit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    def formula(person, period, parameters):
        age = person("age", period)
        return (
            age
            < parameters(period).gov.provinces.nb.benefits.nbcb.eligible_age
        )



from policyengine_canada.model_api import *


class income_tax(Variable):
    value_type = float
    entity = Household
    label = "Income tax after credits"
    unit = CAD
    definition_period = YEAR

    adds = ["income_tax_before_refundable_credits"]
    subtracts = ["refundable_tax_credits"]



from policyengine_canada.model_api import *


class ns_non_refundable_credits(Variable):
    value_type = float
    entity = Person
    label = "NS non refundable credits"
    documentation = "Nova Scotia non refundable tax credits"
    unit = CAD
    definition_period = YEAR

    adds = "gov.provinces.ns.tax.income.credits.non_refundable"



from policyengine_canada.model_api import *


class on_low_income_workers_tax_credit_eligible_people(Variable):
    value_type = float
    entity = Person
    label = (
        "Sum of eligible people for the Ontario Low-Income Workers Tax Credit"
    )
    unit = CAD
    definition_period = YEAR

    adds = ["on_low_income_workers_tax_credit_eligible"]



from policyengine_canada.model_api import *


class gst_credit_reduction(Variable):
    value_type = float
    entity = Household
    label = "GST Credit reduction based on income"
    unit = CAD
    definition_period = YEAR

    # The income concept here is supposed to be based on a slightly modified version of family net income,
    # where you subtract certain income sources such as Registered Disability Savings Plan income. I've ignored
    # these modifications and just used net income.
    def formula(household, period, parameters):
        net_income = household("household_net_income", period)

        reduction = parameters(
            period
        ).gov.cra.tax.income.credits.gst_credit.reduction.calc(net_income)
        # Round to the nearest cent.
        return numpy.around(reduction, 2)



from policyengine_canada.model_api import *


class on_senior_homeowners_property_tax_grant_base(Variable):
    value_type = float
    entity = Person
    label = "Ontario senior homeowners property tax grant base"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.on.tax.grants.oshptg
        age_eligible = person("age", period) >= p.age_eligibility
        amount_if_eligible = min_(person("property_tax", period), p.max_amount)
        return age_eligible * amount_if_eligible



from policyengine_canada.model_api import *


class child_benefit_base(Variable):
    value_type = float
    entity = Household
    label = "Canada Child Benefit Base"
    unit = CAD
    documentation = "Base amount of Canada Child Benefit before reduction."
    definition_period = YEAR

    adds = ["child_benefit_base_person"]



from policyengine_canada.model_api import *


class CanadaWorkersBenefitDisabilityCategory(Enum):
    INELIGIBLE = "Ineligible"
    SINGLE = "Single"
    FAMILY_WITH_ONE_DISABLED_SPOUSE = "Family with one disabled spouse"
    FAMILY_WITH_TWO_DISABLED_SPOUSES = "Family with two disabled spouses"


class cwb_disability_category(Variable):
    value_type = Enum
    entity = Household
    possible_values = CanadaWorkersBenefitDisabilityCategory
    default_value = CanadaWorkersBenefitDisabilityCategory.INELIGIBLE
    label = "Canada workers benefit disability category"
    definition_period = YEAR

    def formula(household, period, parameters):
        person = household.members
        # Compute individual-level eligibility among heads/spouses.
        head_or_spouse = person("is_head_or_spouse", period)
        disability_eligible = person(
            "cwb_disability_supplement_eligible", period
        )
        cwb_family = household("is_cwb_family", period)
        eligible_spouses = household.sum(head_or_spouse * disability_eligible)
        return select(
            [
                eligible_spouses == 2,
                cwb_family & (eligible_spouses == 1),
                eligible_spouses == 1,
            ],
            [
                CanadaWorkersBenefitDisabilityCategory.FAMILY_WITH_TWO_DISABLED_SPOUSES,
                CanadaWorkersBenefitDisabilityCategory.FAMILY_WITH_ONE_DISABLED_SPOUSE,
                CanadaWorkersBenefitDisabilityCategory.SINGLE,
            ],
            default=CanadaWorkersBenefitDisabilityCategory.INELIGIBLE,
        )



from policyengine_canada.model_api import *


class sk_age_amount(Variable):
    value_type = float
    entity = Person
    label = "Saskatchewan age amount credit"
    unit = CAD
    definition_period = YEAR
    reference = (
        "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1sk/td1sk-21e.pdf#page=1",
        "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1sk/td1sk-22e.pdf#page=1",
        "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1sk/td1sk-23e.pdf#page=1",
        "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1-ws/td1-ws-21e.pdf#page=1",
        "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1-ws/td1-ws-22e.pdf#page=1",
        "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1-ws/td1-ws-23e.pdf#page=1",
        "https://publications.saskatchewan.ca/api/v1/products/583/formats/806/download#page=15",
    )
    defined_for = "sk_age_amount_eligible"

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.sk.tax.income.credits.age_amount
        income = person("individual_net_income", period)
        reduction = p.reduction.rate.calc(income)
        return max_(p.max_amount - reduction, 0)



from policyengine_canada.model_api import *


class cwb_disability_supplement_max_amount(Variable):
    value_type = float
    entity = Household
    label = "Eligible for canada workers benefit supplement"
    definition_period = YEAR
    reference = (
        "https://laws-lois.justice.gc.ca/eng/acts/I-3.3/page-89.html#docCont"
    )
    unit = CAD

    def formula(household, period, parameters):
        person = household.members
        eligible = person("cwb_disability_supplement_eligible", period)
        amount = parameters(
            period
        ).gov.cra.benefits.cwb.amount.disability_supplement
        return household.sum(eligible * amount)



from policyengine_canada.model_api import *


class gst_credit(Variable):
    value_type = float
    entity = Household
    label = "GST Credit"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        base_amount = household("gst_credit_base", period)
        boost = household("gst_credit_singles_boost", period)
        reduction = household("gst_credit_reduction", period)
        return max(base_amount + boost - reduction, 0)



from policyengine_canada.model_api import *


class acfb_eligible_children(Variable):
    value_type = int
    entity = Household
    label = "Alberta child and family benefit eligible children"
    definition_period = YEAR

    adds = ["acfb_eligible_child"]



from policyengine_canada.model_api import *


class nu_cost_of_living_credit(Variable):
    value_type = float
    entity = Person
    label = "Nunavut cost of living credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    adds = [
        "nu_cost_of_living_basic_credit",
        "nu_cost_of_living_credit_supplement",
    ]



from policyengine_canada.model_api import *


class full_time_childcare(Variable):
    value_type = bool
    entity = Household
    label = "Full-time child care"
    definition_period = YEAR


# TODO: computation based variable - based on full-time definition



from policyengine_canada.model_api import *


class nb_low_income_tax_reduction(Variable):
    value_type = float
    entity = Household
    label = "New Brunswick low income tax reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    def formula(household, period, parameters):
        p = parameters(
            period
        ).gov.provinces.nb.tax.income.credits.low_income_tax_reduction
        income = household("adjusted_family_net_income", period)
        base = household("nb_low_income_tax_reduction_base", period)
        return max_(0, base - p.rate.calc(income))



import logging
import os
from pathlib import Path
from typing import Dict, Type

from policyengine_canada.entities import entities
from policyengine_canada.data.datasets.country_template_dataset import (
    CountryTemplateDataset,
)
from policyengine_core.data.dataset import Dataset
from policyengine_core.populations.population import Population
from policyengine_core.simulations import (
    Microsimulation as CoreMicrosimulation,
)
from policyengine_core.simulations import Simulation as CoreSimulation
from policyengine_core.taxbenefitsystems import TaxBenefitSystem

from policyengine_canada.variables.household.demographic.geographic.province.in_province import (
    create_10_province_variables,
)

from .constants import COUNTRY_DIR

DATASETS = [CountryTemplateDataset]  # Important: must be instantiated


class CountryTaxBenefitSystem(TaxBenefitSystem):
    entities = entities
    variables_dir = COUNTRY_DIR / "variables"
    parameters_dir = COUNTRY_DIR / "parameters"
    auto_carry_over_input_variables = True
    basic_inputs = [
        "province_name",
        "employment_income",
    ]

    def __init__(self):
        # We initialize our tax and benefit system with the general constructor
        super().__init__(entities)

        self.add_variables(*create_10_province_variables())


system = CountryTaxBenefitSystem()


class Simulation(CoreSimulation):
    default_tax_benefit_system = CountryTaxBenefitSystem
    default_tax_benefit_system_instance = system
    default_role = "parent"
    default_calculation_period = 2023
    default_input_period = 2023


class Microsimulation(CoreMicrosimulation):
    default_tax_benefit_system = CountryTaxBenefitSystem
    default_dataset = CountryTemplateDataset
    default_tax_benefit_system_instance = system
    default_dataset_year = 2023
    default_calculation_period = 2023



from policyengine_canada.model_api import *


class age(Variable):
    value_type = int
    entity = Person
    label = "Age"
    documentation = "Age in years since birth"
    definition_period = YEAR
    default_value = 18



from policyengine_canada.model_api import *


class is_mother(Variable):
    value_type = bool
    entity = Person
    label = "Is a mother"
    definition_period = YEAR

    def formula(person, period, parameters):
        female = person("is_female", period)
        has_children = person("own_children_in_household", period) > 0
        # breastfeeding = person("is_breastfeeding", period)
        # return breastfeeding | (female & has_children)
        return female & has_children



from policyengine_canada.model_api import *


class yt_cftc_disabled_child(Variable):
    value_type = bool
    entity = Person
    label = "Yukon Child Fitness Tax Credit disabled child"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(person, period, parameters):
        disabled = person("is_disabled", period)
        fees = person("physical_activities_fees", period)
        age = person("age", period)
        p = parameters(
            period
        ).gov.provinces.yt.tax.income.credits.childrens_fitness.disability_supplement
        return (fees >= p.minimum_fees) & disabled & (age < p.age)



from policyengine_canada.model_api import *


class child_benefit(Variable):
    value_type = float
    entity = Household
    label = "Canada child benefit"
    unit = CAD
    documentation = (
        "Non taxable amount paid monthly per children under 18 years of age. "
    )
    definition_period = YEAR

    def formula(household, period, parameters):
        base = household("child_benefit_base", period)
        reduction = household("child_benefit_reduction", period)
        return max_(0, base - reduction)



from policyengine_canada.model_api import *


class northern_ontario_energy_credit(Variable):
    value_type = float
    entity = Household
    label = "Northern Ontario energy credit"
    unit = CAD
    definition_period = YEAR
    defined_for = "is_in_northern_ontario"

    def formula(household, period, parameters):
        person = household.members
        age = person("age", period)
        married = household("is_married", period)
        children = household("noec_count_children", period)
        category = household("oeptc_category", period)
        afni = household("adjusted_family_net_income", period)
        p = parameters(period).gov.provinces.on.tax.income.credits.noec
        # Subtract base amount from afni (varies by category).
        excess = max_(afni - p.phase_out.start[category], 0)
        phase_out = excess * p.phase_out.rate
        # Subtract the excess amount from the base noec amount.
        phased_out = p.base[category] - phase_out
        # Single with shared custody household calculation continues - phased-out devided by 2.
        shared_custody_phased_out_divided = (
            phased_out / p.phase_out.shared_custody.divisor
        )
        # Second phase out process - same steps as previously with different amounts.
        shared_custody_excess = max_(
            afni - p.phase_out.shared_custody.start, 0
        )
        shared_custody_second_phase_out = (
            shared_custody_excess * p.phase_out.rate
        )
        shared_custody_second_phased_out = (
            p.phase_out.shared_custody.base - shared_custody_second_phase_out
        )
        shared_custody_second_phased_out_divided = (
            shared_custody_second_phased_out
            / p.phase_out.shared_custody.divisor
        )
        # Single with shared custody household afni under 43_602 amount is 205.50.
        shared_custody_result = where(
            afni > p.phase_out.start[category],
            shared_custody_phased_out_divided
            + shared_custody_second_phased_out_divided,
            p.phase_out.shared_custody.base,
        )
        eligible = (age >= p.age_eligibility) | married | (children > 0)
        amount_if_eligible = max_(
            0,
            where(
                category == category.possible_values.SINGLE_SHARED_CUSTODY,
                shared_custody_result,
                phased_out,
            ),
        )
        return eligible * amount_if_eligible



from policyengine_canada.model_api import *


class household_refundable_tax_credits(Variable):
    value_type = float
    entity = Household
    label = "refundable tax credits"
    definition_period = YEAR
    unit = CAD
    adds = [
        "refundable_tax_credits",  # Federal.
        "on_refundable_credits",  # Ontario.
        "bc_refundable_credits",  # British Columbia.
    ]



from policyengine_canada.model_api import *


class basic_personal_amount_supplement(Variable):
    value_type = float
    entity = Person
    label = "Basic Personal Amount"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        income = person("individual_net_income", period)
        p = parameters(period).gov.cra.tax.income.credits.basic_personal_amount
        max_amount = p.max_amount
        base = p.base
        phase_out_rate = where(
            income > p.phase_out.threshold.start,
            min_(
                (income - p.phase_out.threshold.start)
                / (p.phase_out.threshold.end - p.phase_out.threshold.start),
                1,
            ),
            0,
        )
        return (max_amount - base) - ((max_amount - base) * phase_out_rate)



from policyengine_canada.model_api import *


class ns_low_income_tax_reduction(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia low income tax reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        base = household("ns_low_income_tax_reduction_base", period)
        children = household(
            "ns_low_income_tax_reduction_base_children", period
        )
        reduction = household("ns_low_income_tax_reduction_reduction", period)
        return max_(0, (base + children) - reduction)



from policyengine_canada.model_api import *


class cwb_disability_supplement_phase_in(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit disability supplement phase in"
    definition_period = YEAR
    unit = CAD

    def formula(household, period, parameters):
        income = household("family_working_income", period)
        p = parameters(
            period
        ).gov.cra.benefits.cwb.phase_in.disability_supplement
        return p.calc(income)



from policyengine_canada.model_api import *


class adjusted_family_net_income(Variable):
    value_type = float
    entity = Household
    label = "Adjusted family net income"
    unit = CAD
    documentation = "The family net income minus any universal child care benefit and registered disability savings plan income received"
    definition_period = YEAR

    adds = ["family_net_income"]
    subtracts = [
        "universal_child_care_benefit",
        "registered_disability_savings_plan_income",
    ]



from policyengine_canada.model_api import *


class ns_child_benefit_eligible_children(Variable):
    value_type = int
    entity = Household
    label = "Nova Scotia child benefit eligible children"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    adds = ["ns_child_benefit_eligible_child"]



from policyengine_canada.model_api import *


class nl_physical_activity_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Newfoundland Physical activity tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.NL

    def formula(household, period, parameters):
        person = household.members
        p = parameters(
            period
        ).gov.provinces.nl.tax.income.credits.physical_activity

        # Person has to be either head, spouse or child under 18 to be eligible.
        eligible = (
            person("is_head", period)
            | person("is_spouse", period)
            | (person("age", period) < p.age_eligible)
        )

        individual_expenses = eligible * (
            person("physical_activities_fees", period)
        )

        expenses = household.sum(individual_expenses)
        maximum_amount = min_(expenses, p.max_amount)

        return maximum_amount * p.rate



from policyengine_canada.model_api import *


class ns_age_tax_credit(Variable):
    value_type = float
    entity = Person
    label = "Nova Scotia Age tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(person, period, parameters):
        age = person("age", period)
        income = person("ns_taxable_income", period)
        p = parameters(period).gov.provinces.ns.tax.income.credits.age
        eligibility = (age >= p.age_eligibility) & (
            income < p.income_eligibility
        )

        return eligibility * p.amount



from policyengine_canada.model_api import *


class on_sales_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Ontario sales tax credit"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        base = household("on_sales_tax_credit_base", period)
        reduction = household("on_sales_tax_credit_reduction", period)
        return max_(0, base - reduction)



from policyengine_canada.model_api import *


class cwb_dependant(Variable):
    value_type = bool
    entity = Person
    label = "Canada workers benefit dependant"
    definition_period = YEAR

    def formula(person, period, parameters):
        p = parameters(period).gov.cra.benefits.cwb
        return person("age", period) < p.eligible_age



from policyengine_canada.model_api import *


class bc_family_benefit(Variable):
    value_type = float
    entity = Household
    label = "British Columbia family benefit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(period).gov.provinces.bc.benefits.bcfb
        reduced_amount = household("bc_family_benefit_first_reduction", period)
        second_reduction = p.second_reduction.rate.calc(income)
        return max_(reduced_amount - second_reduction, 0)



from policyengine_canada.model_api import *


class acfb_working_component_reduction(Variable):
    value_type = float
    entity = Household
    label = "Alberta child and family benefit working component reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(household, period, parameters):
        p = parameters(
            period
        ).gov.provinces.ab.benefits.acfb.working_component.phase_out
        income = household("adjusted_family_net_income", period)
        children = household("acfb_eligible_children", period)
        return select(
            # Conditions.
            [children == 1, children == 2, children == 3, children > 3],
            # Results.
            [
                p.one_child.calc(income),
                p.two_children.calc(income),
                p.three_children.calc(income),
                p.four_or_more_children.calc(income),
            ],
            default=0,
        )



from policyengine_canada.model_api import *


class bc_family_benefit_first_reduction(Variable):
    value_type = float
    entity = Household
    label = "British Columbia family benefit first reduction"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        children = household("bc_family_benefit_eligible_children", period)
        p = parameters(period).gov.provinces.bc.benefits.bcfb.first_reduction
        base = household("bc_family_benefit_base", period)
        reduction = p.rate.calc(income)
        # The maximum family benefit is reduced to an amount dependant on the amount of children.
        # Each child has a designated amount, varying for the first two children.
        reduction_cap = (
            ((children > 0) * p.max_amount.one_child)
            + ((children > 1) * p.max_amount.two_children)
            + (max_(children - 2, 0) * p.max_amount.three_or_more_children)
        )
        return max_(max_(base - reduction, reduction_cap), 0)



from policyengine_canada.model_api import *


class qc_fa_payment(Variable):
    value_type = float
    entity = Household
    label = "Quebec family allowance payment amount"
    reference = "https://www.legisquebec.gouv.qc.ca/en/document/cs/I-3?langCont=en#se:1029_8_61_18"
    definition_period = YEAR
    defined_for = "qc_fa_eligibility"

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.fa

        # check if the household has cohabiting spouse
        has_spouse = household("is_married", period)

        # eligible dependent child - under 18 years of age
        person = household.members
        age_eligible = person("age", period) < p.age_eligibility

        # check if each child is full custody
        full_custody = person("full_custody", period)
        shared_custody_multiplier = where(
            full_custody, 1, p.shared_custody_multiplier
        )

        # method 1: calculate using max amount with reduction threshold
        # (C + D) − 4% (E − F) in taxation axt
        maximum_child_amount = household.sum(
            age_eligible * p.child_amount.max * shared_custody_multiplier
        )

        single_parent_max_amount = where(
            has_spouse, 0, p.single_parent_amount.max
        )

        income = household("adjusted_family_net_income", period)
        reduction = where(
            has_spouse,
            p.reduction.two_parent_family.calc(income),
            p.reduction.single_parent_family.calc(income),
        )
        max_credit_amount = (
            maximum_child_amount + single_parent_max_amount - reduction
        )

        # method 2: calculate using min amounts, G+H in taxation axt
        minimum_child_amount = household.sum(
            age_eligible * p.child_amount.min * shared_custody_multiplier
        )
        single_parent_min_amount = where(
            has_spouse, 0, p.single_parent_amount.min
        )

        min_credit_amount = minimum_child_amount + single_parent_min_amount

        return max_(min_credit_amount, max_credit_amount)



from policyengine_canada.model_api import *


class on_child_benefit_base(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Benefit Base"
    unit = CAD
    documentation = "Base amount of Ontario Child Benefit before reduction."
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(household, period, parameters):
        children = household("child_benefit_eligible_children", period)
        p = parameters(period).gov.provinces.on.benefits.ocb
        return children * p.base



from policyengine_canada.model_api import *


class non_refundable_tax_credits(Variable):
    value_type = float
    entity = Person
    label = "Non-refundable tax credits"
    unit = CAD
    definition_period = YEAR

    adds = "gov.cra.tax.income.credits.non_refundable"



from policyengine_canada.model_api import *


class bc_family_benefit_base(Variable):
    value_type = float
    entity = Household
    label = "British Columbia family benefit base"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(household, period, parameters):
        children = household("bc_family_benefit_eligible_children", period)
        p = parameters(period).gov.provinces.bc.benefits.bcfb.base
        amount_if_eligible = (
            ((children > 0) * p.one_child)
            + ((children > 1) * p.two_children)
            + (max_(children - 2, 0) * p.three_or_more_children)
        )
        return amount_if_eligible



from policyengine_canada.model_api import *


class qc_cce_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Quebec childcare expenses tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    adds = [
        "qc_cce_disabled_child_tax_credit",
        "qc_cce_nondisabled_child_tax_credit",
    ]



from policyengine_canada.model_api import *


class sk_low_income_credit_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Sasktachewan low income tax credit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    def formula(person, period, parameters):
        age = person("age", period)
        return (
            age
            < parameters(
                period
            ).gov.provinces.sk.tax.income.credits.slic.child.ineligible_age
        )



from policyengine_canada.model_api import *


class ns_child_benefit(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia Child Benefit"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        children = household("ns_child_benefit_eligible_children", period)
        p = parameters(period).gov.provinces.ns.benefits.child_benefit
        income = household("adjusted_family_net_income", period)
        eligible = income < p.upper_threshold
        # In the lower income bracket, the families are eligible to receive
        # $1,275 per child.
        lower_income_amount = children * p.base
        # In the higher income bracket, families are eligible to receive
        # $1,275 for the first born child.
        higher_income_base = p.base * (children > 0)
        # In the higher income bracket, families are eligible to receive
        # $637.50 for each child after the first born.
        higher_income_additional_child = max_(
            0, p.higher_income_base * (children - 1)
        )
        amount_if_eligible = where(
            income < p.lower_threshold,
            lower_income_amount,
            higher_income_base + higher_income_additional_child,
        )
        return where(eligible, amount_if_eligible, 0)



from policyengine_canada.model_api import *


class cwb_disability_supplement_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for canada workers benefit supplement"
    definition_period = YEAR
    reference = (
        "https://laws-lois.justice.gc.ca/eng/acts/I-3.3/page-89.html#docCont"
    )

    def formula(person, period, parameters):
        # For now, apply a person-level logic based on age.
        eligible_person = ~person("cwb_dependant", period)
        disability_eligible = person("dtc_eligible", period)
        return eligible_person & disability_eligible



from policyengine_canada.model_api import *


class yt_childrens_fitness_tax_credit_disability_supplement(Variable):
    value_type = float
    entity = Household
    label = "Yukon children fitness tax credit disability supplement"
    definition_period = YEAR
    unit = CAD
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        disabled_children = household("yt_cftc_disabled_children", period)
        p = parameters(
            period
        ).gov.provinces.yt.tax.income.credits.childrens_fitness.disability_supplement
        return disabled_children * p.base



from policyengine_canada.model_api import *


class oeptc_property_tax_component(Variable):
    value_type = float
    entity = Household
    label = "Oeptc property tax component"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        # Varies by senior status, not household type.
        senior_status = household("oeptc_senior_status", period)
        p = parameters(
            period
        ).gov.provinces.on.tax.income.credits.oeptc.property_tax_component
        # Step A: "Multiply the result of Step 1 by x%"
        occupancy_cost = household("oeptc_occupancy_cost", period)
        multiplied_occupancy_cost = occupancy_cost * p.multiplier
        # Step B: Use A or $x, whichever is less.
        cap = p.initial_cap[senior_status]
        capped_multiplied_occupancy_cost = min_(multiplied_occupancy_cost, cap)
        # Step C: Add $x to B.

        return min_(
            occupancy_cost,
            capped_multiplied_occupancy_cost + p.supplement[senior_status],
        )



from policyengine_canada.model_api import *


class is_in_northern_ontario(Variable):
    value_type = bool
    entity = Household
    label = "Is located in Northern Ontario"
    definition_period = YEAR
    defined_for = ProvinceCode.ONT


# placeholder until district enum is made



from policyengine_canada.model_api import *


class benefits(Variable):
    value_type = float
    entity = Household
    label = "benefits"
    unit = CAD
    definition_period = YEAR
    adds = [
        "child_benefit",
        "child_disability_benefit",
        "canada_workers_benefit",
        "dental_benefit",
        "oas_net",
        # Ontario programs.
        "on_benefits",
        # British Columbia programs.
        "bc_benefits",
        # New Brunswick programs.
        "nb_benefits",
        # Saskatchewan programs.
        "sk_benefits",
    ]



from policyengine_canada.model_api import *


class dental_expenses(Variable):
    value_type = float
    entity = Person
    label = "Costs for dental care"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class ClimateActionIncentiveCategory(Enum):
    HEAD = "Head"
    SPOUSE = "Spouse"
    ELDEST_CHILD_IN_SINGLE_PARENT_HOUSEHOLD = (
        "Eldest child in single parent household"
    )
    OTHER_CHILD = "Other child"


class climate_action_incentive_category(Variable):
    value_type = Enum
    entity = Person
    possible_values = ClimateActionIncentiveCategory
    default_value = ClimateActionIncentiveCategory.HEAD
    definition_period = YEAR

    def formula(person, period, parameters):
        is_single_parent_household = person.household(
            "climate_action_incentive_single_parent_household", period
        )
        eldest_child = person(
            "is_eldest_child_for_climate_action_incentive", period
        )
        eldest_child_in_single_parent_household = (
            is_single_parent_household & eldest_child
        )
        other_child = (
            person("is_child_for_climate_action_incentive", period)
            & ~eldest_child_in_single_parent_household
        )
        return select(
            [
                person("is_head", period),
                person("is_spouse", period),
                eldest_child_in_single_parent_household,
                other_child,
            ],
            [
                ClimateActionIncentiveCategory.HEAD,
                ClimateActionIncentiveCategory.SPOUSE,
                ClimateActionIncentiveCategory.ELDEST_CHILD_IN_SINGLE_PARENT_HOUSEHOLD,
                ClimateActionIncentiveCategory.OTHER_CHILD,
            ],
        )



from policyengine_canada.model_api import *


class sk_disability_amount(Variable):
    value_type = float
    entity = Person
    unit = CAD
    label = "Saskatchewan disability amount tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.provinces.sk.tax.income.credits.disability_amount
        disabled = person("is_disabled", period)
        return disabled * p.amount



from policyengine_canada.model_api import *


class noec_count_children(Variable):
    value_type = int
    entity = Household
    label = "Northern Ontario Energy Tax Credit children"
    definition_period = YEAR

    adds = ["noec_child"]



from policyengine_canada.model_api import *


class count_dependants(Variable):
    value_type = int
    entity = Household
    label = "Dependannts"
    unit = CAD
    documentation = "Number of dependants"
    definition_period = YEAR
    adds = ["is_dependant"]



from policyengine_canada.model_api import *


class yt_first_nations_tax(Variable):
    value_type = float
    entity = Household
    label = "Yukon first nations tax"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(
            period
        ).gov.provinces.yt.tax.income.credits.first_nations_tax
        eligible = household("resides_on_settlement_land", period)
        return income * p.rate * eligible



from policyengine_canada.model_api import *


class ab_pension_credit(Variable):
    value_type = float
    entity = Person
    label = "Alberta Pension Credit"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.ab.tax.income.credits.pension
        pension_income = person("pension_and_savings_plan_income", period)
        return min_(pension_income, p.max_amount)



from policyengine_canada.model_api import *


class qc_cce_disabled_child_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Quebec disabled child childcare expenses tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.cce

        income = household("adjusted_family_net_income", period)
        credit_rate = p.rate.calc(income)

        person = household.members

        expense = person("childcare_expense", period)

        individual_income = person("individual_net_income", period)
        own_child = person("is_child_of_filer", period)
        dependant = person("is_dependant", period)
        disabled = person("is_disabled", period)

        qualifying_non_own_child_dependant = dependant & (
            individual_income <= p.child_income_limit
        )
        eligible = disabled & (own_child | qualifying_non_own_child_dependant)

        credit = (
            credit_rate
            * eligible
            * min_(p.disabled_child_expense_limit, expense)
        )

        return household.sum(credit)



from policyengine_canada.model_api import *


class mb_pension_amount_credit(Variable):
    value_type = float
    entity = Person
    label = "Manitoba pension amount credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.MB

    def formula(person, period, parameters):
        pension_and_savings_income = person(
            "pension_and_savings_plan_income", period
        )
        max_amount = parameters(
            period
        ).gov.provinces.mb.tax.income.credits.pension_amount.max_amount
        return min_(pension_and_savings_income, max_amount)



from policyengine_canada.model_api import *


class sk_senior_supplementary_credit_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Sasktachewan senior supplementary tax credit eligibility"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.provinces.sk.tax.income.credits.sk_senior_supplementary
        age = person("age", period)
        return age >= p.age_threshold



from policyengine_canada.model_api import *


class OEPTCSeniorStatus(Enum):
    SENIOR = "Has at least one senior"
    NON_SENIOR = "Has no seniors"


class oeptc_senior_status(Variable):
    value_type = Enum
    entity = Household
    label = "Is a senior for the OEPTC"
    possible_values = OEPTCSeniorStatus
    default_value = OEPTCSeniorStatus.NON_SENIOR
    definition_period = YEAR

    def formula(household, period, parameters):
        oldest_age = household.max(household.members("age", period))
        p = parameters(period).gov.provinces.on.tax.income.credits.oeptc
        return where(
            oldest_age >= p.senior_age,
            OEPTCSeniorStatus.SENIOR,
            OEPTCSeniorStatus.NON_SENIOR,
        )



from policyengine_canada.model_api import *


class total_individual_pre_tax_income(Variable):
    value_type = float
    entity = Person
    label = "Total Individual Pre Tax Income"
    unit = CAD
    documentation = "Sum of the 5 granular income types, IE employment, self-employment, investment, benefits, and pension-and-savings-plan"
    definition_period = YEAR
    reference = "https://www.canada.ca/en/financial-consumer-agency/services/financial-toolkit/taxes/taxes-2/4.html"

    adds = [
        "benefits_income",
        "employment_income",
        "investment_income",
        "pension_and_savings_plan_income",
        "self_employment_income",
    ]



from policyengine_canada.model_api import *


class on_child_care_fee_subsidy_reduction(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Care Fee Subsidy reduction"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        children = household("on_child_care_fee_subsidy_children", period)
        eligible_children = household(
            "on_child_care_fee_subsidy_reduction_children", period
        )
        full_time = household("full_time_childcare", period)
        reduction_factor = parameters(
            period
        ).gov.provinces.on.subsidies.on_child_care_fee_subsidy.reduction.factor
        full_factor = where(
            children > 0, eligible_children * reduction_factor / children, 0
        )
        full_time_reduction = household(
            "on_child_care_fee_subsidy_full_time", period
        )
        part_time_reduction = household(
            "on_child_care_fee_subsidy_part_time", period
        )
        eligible = children > 0
        return (
            eligible
            * full_factor
            * where(full_time, full_time_reduction, part_time_reduction)
        )



from policyengine_canada.model_api import *


class Province(Enum):
    UNKNOWN = "Unknown"
    ALBERTA = "Alberta"
    BRITISH_COLUMBIA = "British Columbia"
    MANITOBA = "Manitoba"
    NEW_BRUNSWICK = "New Brunswick"
    NEWFOUNDLAND_AND_LABRADOR = "Newfoundland and Labrador"
    NOVA_SCOTIA = "Nova Scotia"
    NORTHWEST_TERRITORIES = "Northwest Territories"
    NUNAVUT = "Nunavut"
    ONTARIO = "Ontario"
    PRINCE_EDWARD_ISLAND = "Prince Edward Island"
    QUEBEC = "Quebec"
    SASKATCHEWAN = "Saskatchewan"
    YUKON = "Yukon"


class province(Variable):
    value_type = Enum
    possible_values = Province
    default_value = Province.UNKNOWN
    entity = Household
    label = "Province"
    definition_period = YEAR



from policyengine_canada.model_api import *


class yt_childrens_fitness_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Yukon children fitness tax credit"
    definition_period = YEAR
    unit = CAD
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        total_children = household("yt_cftc_eligible_children", period)
        person = household.members
        fees = person("physical_activities_fees", period)
        disability_supplement = household(
            "yt_childrens_fitness_tax_credit_disability_supplement", period
        )
        p = parameters(
            period
        ).gov.provinces.yt.tax.income.credits.childrens_fitness
        # maximum of $ 1,000 per child tof eligible fees can be multiplied by
        # the refundable rate of % 6.4
        maximum_fees = total_children * p.base
        refundable_portion = fees * p.rate
        per_supplement_amount = min_(refundable_portion, maximum_fees)
        return per_supplement_amount + disability_supplement



from policyengine_canada.model_api import *


class GSTCreditCategory(Enum):
    HEAD = "Head"
    SPOUSE = "Spouse"
    CHILD = "Child"


class gst_credit_category(Variable):
    value_type = Enum
    entity = Person
    possible_values = GSTCreditCategory
    default_value = GSTCreditCategory.HEAD
    definition_period = YEAR

    def formula(person, period, parameters):
        return select(
            [
                person("is_head", period),
                # Eldest children of single parents are treated as spouses
                # for GST credit purposes.
                person("is_spouse", period)
                | person(
                    "is_eldest_child_in_single_household_for_gst_credit",
                    period,
                ),
            ],
            [GSTCreditCategory.HEAD, GSTCreditCategory.SPOUSE],
            default=GSTCreditCategory.CHILD,
        )



from policyengine_canada.model_api import *


class yt_childrens_arts_credit_expenses(Variable):
    value_type = float
    entity = Household
    label = "Yukon childrens arts credit expenses"
    definition_period = YEAR
    defined_for = ProvinceCode.YT



from policyengine_canada.model_api import *


class child_benefit_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Canada Child Benefit"
    definition_period = YEAR

    def formula(person, period, parameters):
        return person("child_benefit_base_person", period) > 0



from policyengine_canada.model_api import *


class ns_income_assistance(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia Income Assistance"
    unit = CAD
    definition_period = YEAR
    reference = "https://novascotia.ca/coms/employment/income_assistance/Eligibility.html#19"
    defined_for = ProvinceCode.NS



from policyengine_canada.model_api import *


class child_benefit_base_person(Variable):
    value_type = float
    entity = Person
    label = "Canada Child Benefit Base"
    unit = CAD
    documentation = "Base amount of Canada Child Benefit before reduction."
    definition_period = YEAR

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(period).gov.cra.benefits.ccb
        full_custody = person("full_custody", period)
        return where(
            full_custody, p.base.calc(age), p.base.calc(age) / p.divisor
        )



from policyengine_canada.model_api import *


class childcare_received_days(Variable):
    value_type = int
    entity = Household
    label = "Received days of child care in a year"
    unit = "day"
    definition_period = YEAR



from policyengine_canada.model_api import *


class yt_cftc_disabled_children(Variable):
    value_type = int
    entity = Household
    label = "Yukon Fitness Tax Credit Disabled Children"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    adds = ["yt_cftc_disabled_child"]



from policyengine_canada.model_api import *


class sk_low_income_credit_eligible_children(Variable):
    value_type = int
    entity = Household
    label = "Sasktachewan low income tax credit eligible children"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    adds = ["sk_low_income_credit_eligible_child"]



from policyengine_canada.model_api import *


class yt_child_benefit_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Yukon child benefit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(person, period, parameters):
        return person("is_dependant", period)



from policyengine_canada.model_api import *


class nu_child_benefit_base_component_base(Variable):
    value_type = float
    entity = Household
    label = "Nunvaut child benefit base component base"
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(household, period, parameters):
        children = household("nu_child_benefit_eligible_children", period)
        return (
            children
            * parameters(period).gov.provinces.nu.tax.benefits.nucb.base.amount
        )



from policyengine_canada.model_api import *


class is_child_for_climate_action_incentive(Variable):
    value_type = bool
    entity = Person
    label = (
        "Is a for the climate action incentive eligible child in a Household"
    )
    definition_period = YEAR
    defined_for = "is_dependant"

    def formula(person, period, parameters):
        adult_age = parameters(
            period
        ).gov.cra.tax.income.credits.climate_action_incentive.adult_age
        return person("age", period) < adult_age
        # The definition of an eligible child also includes living with
        # parents and eligibilty under CCB. These are not included here.



from policyengine_canada.model_api import *


class qc_sa_spouse_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Quebec senior assistance tax credits eligible senior's spouse"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.sa

        age_eligible = person("age", period) >= p.age_eligibility
        spouse = person("is_spouse", period)

        return spouse & age_eligible



from policyengine_canada.model_api import *


class qc_basic_personal_amount(Variable):
    value_type = float
    entity = Person
    label = "Quebec basic personal amount"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.QC
    reference = (
        "https://www.revenuquebec.ca/documents/en/formulaires/tp/2022-12/TP-1.D-V%282022-12%29.pdf",
        "https://www.revenuquebec.ca/en/citizens/income-tax-return/completing-your-income-tax-return/how-to-complete-your-income-tax-return/line-by-line-help/350-to-398-1-non-refundable-tax-credits/line-350/",
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits
        return p.basic_personal_amount



from policyengine_canada.model_api import *


class oeptc_occupancy_cost(Variable):
    value_type = float
    entity = Household
    label = "OEPTC occupancy cost"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        person = household.members
        property_tax = person("property_tax", period)
        p = parameters(
            period
        ).gov.provinces.on.tax.income.credits.oeptc.occupancy_costs
        student_residence = household("lived_in_a_student_residence", period)
        countable_rent = person("rent", period) * p.rent_multiplier
        student_residence_addition = (
            student_residence * p.student_resident_supplement
        )
        return household.sum(
            countable_rent + property_tax + student_residence_addition
        )



from policyengine_canada.model_api import *


class bc_non_refundable_credits(Variable):
    value_type = float
    entity = Person
    label = "British Columbia non-refundable credits"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC
    adds = "gov.provinces.bc.tax.income.credits.non_refundable"



from policyengine_canada.model_api import *


class additional_dental_benefit(Variable):
    value_type = float
    entity = Household
    label = "Additional dental benefit"
    unit = CAD
    definition_period = YEAR
    documentation = "If dental costs exceed a threshold, a household may apply for an additional payment."



from policyengine_canada.model_api import *


class ab_age_credit_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for the Alberta age credit"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(person, period, parameters):
        age = person("age", period)
        return (
            age
            >= parameters(
                period
            ).gov.provinces.ab.tax.income.credits.age_credit.eligible_age
        )



from policyengine_canada.model_api import *


class on_child_care_fee_subsidy_reduction_children(Variable):
    value_type = int
    entity = Household
    label = "Ontario Child Care Fee Subsidy reduction children"
    unit = CAD
    definition_period = YEAR

    adds = ["is_child_for_on_child_care_fee_subsidy_reduction"]



from policyengine_canada.model_api import *


class nu_child_benefit_base_component(Variable):
    value_type = float
    entity = Household
    label = "Nunvaut child benefit base component "
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(household, period, parameters):
        base = household("nu_child_benefit_base_component_base", period)
        reduction = household(
            "nu_child_benefit_base_component_reduction", period
        )
        return max_(0, base - reduction)



from policyengine_canada.model_api import *


class gst_credit_dependant_children(Variable):
    value_type = int
    entity = Household
    label = "GST credit dependant Children"
    unit = CAD
    documentation = "Number of eligible dependant children"
    definition_period = YEAR
    adds = ["is_child_for_gst_credit"]



from policyengine_canada.model_api import *


class sk_taxable_income(Variable):
    value_type = float
    entity = Person
    label = "Saskatchewan taxable income"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.revenue.pa.gov/FormsandPublications/FormsforIndividuals/PIT/Documents/2021/2021_pa-40in.pdf#page=8"
    defined_for = ProvinceCode.SK
    adds = ["total_individual_pre_tax_income"]



from policyengine_canada.model_api import *


class childrens_sport_and_culture_participation_costs(Variable):
    value_type = int
    entity = Household
    unit = CAD
    label = " Childrens Sport and Culture Participation Costs"
    definition_period = YEAR



from policyengine_core.model_api import *
from policyengine_canada.entities import *
from policyengine_canada.variables.household.demographic.geographic.province.province_code import (
    ProvinceCode,
)
from policyengine_canada.tools.general import *


CAD = "currency-CAD"


PROVINCES = [
    "AB",
    "BC",
    "MB",
    "NB",
    "NL",
    "NS",
    "NT",
    "NU",
    "ONT",
    "PE",
    "QC",
    "SK",
    "YT",
]



from policyengine_canada.model_api import *


class bc_climate_action_tax_credit_person(Variable):
    value_type = float
    entity = Person
    label = "British Columbia Climate Action amount per individual"
    unit = CAD
    documentation = "Determination of the amount per individual"
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(person, period, parameters):
        category = person("bc_climate_action_incentive_category", period)
        amounts = parameters(
            period
        ).gov.provinces.bc.tax.income.credits.bccatc.amount
        return amounts[category]



from policyengine_canada.model_api import *


class household_net_income(Variable):
    value_type = float
    entity = Household
    label = "net income"
    unit = CAD
    definition_period = YEAR
    adds = ["market_income", "refundable_tax_credits", "benefits"]
    subtracts = ["household_income_tax_before_refundable_credits"]



from policyengine_canada.model_api import *


class on_trillium_benefit(Variable):
    value_type = float
    entity = Household
    label = "Ontario Trillium Benefit"
    documentation = "Sum of the Ontario energy and property tax credit, the Northern Ontario energy credit, and the Ontario sales tax credit programs."
    definition_period = YEAR

    adds = ["oeptc", "on_sales_tax_credit", "northern_ontario_energy_credit"]



from policyengine_canada.model_api import *


class cwb_base(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit base amount"
    definition_period = YEAR
    unit = CAD

    def formula(household, period, parameters):
        max_amount = household("cwb_base_max_amount", period)
        phase_in = household("cwb_base_phase_in", period)
        phase_out = household("cwb_base_phase_out", period)
        max_amount_after_phase_in = min_(max_amount, phase_in)
        return max_(0, max_amount_after_phase_in - phase_out)



from policyengine_canada.model_api import *


class income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = Person
    label = "Income tax before refundable credits"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        income_tax_before_credits = person("income_tax_before_credits", period)
        non_refundable_tax_credits = person(
            "non_refundable_tax_credits", period
        )
        return max_(income_tax_before_credits - non_refundable_tax_credits, 0)



from policyengine_canada.model_api import *


class on_child_care_fee_subsidy_part_time(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Care Fee Subsidy in part time care"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        childcare_expenses = household("childcare_costs", period)
        days = household("childcare_received_days", period)
        multiplier = parameters(
            period
        ).gov.provinces.on.subsidies.on_child_care_fee_subsidy.part_time_child_care_multiplier
        eligible = days > 0
        amount_if_eligible = childcare_expenses / multiplier
        return eligible * amount_if_eligible



from policyengine_canada.model_api import *


class qc_cce_nondisabled_child_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Quebec nondisabled child childcare expenses tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.cce

        income = household("adjusted_family_net_income", period)
        credit_rate = p.rate.calc(income)

        person = household.members

        expense = person("childcare_expense", period)
        age = person("age", period)

        expense_limit = p.nondisabled_child_expense_limit.calc(age)

        credit = credit_rate * min_(expense_limit, expense)

        return household.sum(credit)



from policyengine_canada.model_api import *


class OEPTCCategory(Enum):
    MARRIED = "Married"
    SINGLE_NO_CHILDREN = "Single with no children"
    SINGLE_SHARED_CUSTODY = "Single with shared custody of children"
    SINGLE_SOLE_CUSTODY = "Senior with sole custody of children"


class oeptc_category(Variable):
    value_type = Enum
    entity = Household
    possible_values = OEPTCCategory
    default_value = OEPTCCategory.SINGLE_SHARED_CUSTODY
    definition_period = YEAR

    def formula(household, period, parameters):
        married = household("is_married", period)
        children = household("oeptc_count_children", period)
        has_any_full_custody_children = (
            add(household, period, ["full_custody"]) > 0
        )
        return select(
            [married, children == 0, has_any_full_custody_children],
            [
                OEPTCCategory.MARRIED,
                OEPTCCategory.SINGLE_NO_CHILDREN,
                OEPTCCategory.SINGLE_SOLE_CUSTODY,
            ],
            default=OEPTCCategory.SINGLE_SHARED_CUSTODY,
        )



from policyengine_canada.model_api import *


class canada_workers_benefit(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit"
    definition_period = YEAR
    unit = CAD

    adds = ["cwb_base", "cwb_disability_supplement"]



from policyengine_canada.model_api import *


class acfb_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Alberta child and family benefit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.ab.benefits.acfb
        age = person("age", period)
        return age < p.age_eligibility



from policyengine_canada.model_api import *


class qc_fa_credit(Variable):
    value_type = float
    entity = Household
    label = "Quebec family allowance credit"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    adds = ["qc_fa_payment", "qc_fa_supplement"]



from policyengine_canada.model_api import *


class refundable_tax_credits(Variable):
    value_type = float
    entity = Household
    label = "refundable tax credits"
    unit = CAD
    definition_period = YEAR

    adds = "gov.cra.tax.income.credits.refundable"



from policyengine_canada.model_api import *


class adult_years_in_canada(Variable):
    value_type = int
    entity = Person
    label = "Adult Years In Canada"
    unit = CAD
    documentation = "Years spent in Canada after the age of 18"
    definition_period = YEAR



from policyengine_canada.model_api import *


class alberta_child_and_family_benefit(Variable):
    value_type = float
    entity = Household
    label = "Alberta child and family benefit"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    adds = ["acfb_base_component", "acfb_working_component"]



from policyengine_canada.model_api import *


class climate_action_incentive_dependant_children(Variable):
    value_type = int
    entity = Household
    label = "Climate action incentive dependant Children"
    unit = CAD
    documentation = "Number of eligible dependant children"
    definition_period = YEAR

    adds = ["is_child_for_climate_action_incentive"]



from policyengine_canada.model_api import *


class basic_personal_amount(Variable):
    value_type = float
    entity = Person
    label = "Basic Personal Amount"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        p = parameters(period).gov.cra.tax.income.credits.basic_personal_amount
        supplement = person("basic_personal_amount_supplement", period)
        return p.base + supplement



from policyengine_canada.model_api import *


class nb_child_benefit_eligible_children(Variable):
    value_type = int
    entity = Household
    label = "New Brunswick child benefit eligible children"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    adds = ["nb_child_benefit_eligible_child"]



from policyengine_canada.model_api import *


class physical_activities_fees(Variable):
    value_type = float
    entity = Person
    label = "Fees for physical activities"
    definition_period = YEAR
    unit = CAD



from policyengine_canada.model_api import *


class bc_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = Person
    label = "British Columbia income tax before refundable credits"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/frequently-asked-questions-individuals/canadian-income-tax-rates-individuals-current-previous-years.html"
    defined_for = ProvinceCode.BC

    def formula(person, period, parameters):
        income_tax_before_credits = person(
            "bc_income_tax_before_credits", period
        )
        non_refundable_tax_credits = person(
            "bc_non_refundable_credits", period
        )
        return max_(income_tax_before_credits - non_refundable_tax_credits, 0)



from policyengine_canada.model_api import *


class ns_affordable_living_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "NSALTC"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        children = household("count_children", period)
        p = parameters(period).gov.provinces.ns.tax.income.credits.nsaltc
        child_amount = p.child_base * children
        max_amount = p.individual_base + child_amount
        reduction = p.reduction.calc(income)
        return max_(max_amount - reduction, 0)



from policyengine_canada.model_api import *


class gst_credit_single_parent_household(Variable):
    value_type = bool
    entity = Household
    label = "GST Credit single parent status"
    unit = CAD
    documentation = "Determination wether the filer is a single parent eligible for the full boost amount in the GST credit."
    definition_period = YEAR

    def formula(household, period, parameters):
        married = household("is_married", period)
        gst_credit_children = household(
            "gst_credit_dependant_children", period
        )
        return ~married & (gst_credit_children > 0)



from policyengine_canada.model_api import *


class training_credit_income(Variable):
    value_type = float
    entity = Person
    label = "Canada Training Credit Income"
    unit = CAD
    documentation = "The sum of employment income, self-employment income, maternity benefits and parental benefits"
    definition_period = YEAR

    adds = "gov.cra.tax.income.credits.training.income_sources"



from policyengine_canada.model_api import *


class qc_sa_married_both_eligible(Variable):
    value_type = int
    entity = Household
    label = "Quebec senior assistance tax credits eligible senior couple"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.sa
        person = household.members

        age_eligible = person("age", period) >= p.age_eligibility
        spouse_eligible = person("qc_sa_spouse_eligible", period)

        eligible = age_eligible & spouse_eligible
        return household.sum(eligible)



from policyengine_canada.model_api import *


class bc_tax_reduction_credit(Variable):
    value_type = float
    entity = Person
    label = "British Columbia tax reduction credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(person, period, parameters):
        income = person("bc_taxable_income", period)
        p = parameters(
            period
        ).gov.provinces.bc.tax.income.credits.tax_reduction
        return max_(p.base - p.reduction.calc(income), 0)



from policyengine_core.model_api import *
from policyengine_canada.entities import *


def in_province(province):
    def is_eligible(population, period, parameters):
        return population("province_code_str", period) == province

    return is_eligible



from policyengine_canada.model_api import *


class qc_fa_supplement(Variable):
    value_type = float
    entity = Household
    label = "Quebec family allowance supplement"
    reference = "https://www.legisquebec.gouv.qc.ca/en/document/cs/I-3?langCont=en#se:1029_8_61_18"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.fa

        person = household.members

        # check if the child is full custody of the head
        full_custody = person("full_custody", period)
        shared_custody_multiplier = where(
            full_custody, 1, p.shared_custody_multiplier
        )

        # Supplement for Handicapped Children
        handicapped = person("is_disabled", period)
        supplement_handicapped = (
            handicapped * p.supplement.handicapped_child.base_amount
        )

        # Supplement for the Purchase of School Supplies
        age = person("age", period)
        supplement_school_supplies = where(
            handicapped,
            p.supplement.school_supplies.handicapped_child.calc(age),
            p.supplement.school_supplies.non_handicapped_child.calc(age),
        )

        # Supplement for Handicapped Children Requiring Exceptional Care
        handicapped_tier1 = person("qc_fa_exceptional_care_tier1", period)
        supplement_handicapped_tier1 = (
            handicapped
            * handicapped_tier1
            * p.supplement.handicapped_child.exceptional_care_amount.tier1
        )

        handicapped_tier2 = person("qc_fa_exceptional_care_tier2", period)
        supplement_handicapped_tier2 = (
            handicapped
            * handicapped_tier2
            * p.supplement.handicapped_child.exceptional_care_amount.tier2
        )

        supplements = shared_custody_multiplier * (
            supplement_school_supplies
            + supplement_handicapped
            + supplement_handicapped_tier1
            + supplement_handicapped_tier2
        )

        return household.sum(supplements)



from policyengine_canada.model_api import *


class cwb_base_phase_out(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit base phase out"
    definition_period = YEAR
    unit = CAD

    def formula(household, period, parameters):
        afni = household("adjusted_family_net_income", period)
        p = parameters(period).gov.cra.benefits.cwb.phase_out.base
        family = household("is_cwb_family", period)
        return where(family, p.family.calc(afni), p.single.calc(afni))



from policyengine_canada.model_api import *


class ns_child_benefit_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Nova Scotia child benefit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(period).gov.provinces.ns.benefits.child_benefit
        return age < p.age_eligibility



from policyengine_canada.model_api import *


class nb_age_amount_credit(Variable):
    value_type = float
    entity = Person
    label = "New Brunswick age amount credit"
    definition_period = YEAR
    defined_for = ProvinceCode.NB

    def formula(person, period, parameters):
        age = person("age", period)
        income = person("individual_net_income", period)
        p = parameters(period).gov.provinces.nb.tax.income.credits.age_amount
        eligible = age >= p.age_eligibility
        reduced_amount = p.amount - p.reduction_rate.calc(income)
        return eligible * max_(0, reduced_amount)



from policyengine_canada.model_api import *


class rent_paid_to_public_or_non_profit_long_term_care_home(Variable):
    value_type = float
    entity = Household
    label = "Amounts paid for a public or non-profit long term care home"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class registered_disability_savings_plan_income(Variable):
    value_type = float
    entity = Person
    label = "Registered disability savings plan income"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class climate_action_incentive_single_parent_household(Variable):
    value_type = bool
    entity = Household
    label = "Canada Climate Action single parent status"
    unit = CAD
    documentation = "Determination wether the filer is a single parent eligible for the climate action incentive"
    definition_period = YEAR

    def formula(household, period, parameters):
        married = household("is_married", period)
        cai_children = household(
            "climate_action_incentive_dependant_children", period
        )
        return ~married & (cai_children > 0)



from policyengine_canada.model_api import *


class qc_fa_exceptional_care_tier2(Variable):
    value_type = bool
    entity = Person
    label = "Quebec family allowance handicapped children requiring Tier 2 exceptional care"
    reference = "https://www.rrq.gouv.qc.ca/en/enfants/enfant_handicape/seh-necessitant-soins-exceptionnels/Pages/seh-necessitant-soins-exceptionnels.aspx"
    definition_period = YEAR
    defined_for = ProvinceCode.QC



from policyengine_canada.model_api import *


class nu_cost_of_living_credit_adjusted_net_income(Variable):
    value_type = float
    entity = Person
    label = "Nunavut adjusted income after armed force and personal deduction"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(person, period, parameters):
        income = person("individual_net_income", period)
        armed_deduction = person(
            "canadian_armed_forces_and_personnel_deduction", period
        )
        return max_(0, income - armed_deduction)



from policyengine_canada.model_api import *


class is_cwb_family(Variable):
    value_type = bool
    entity = Household
    label = "Is a family for the Canada Workers Benefit"
    definition_period = YEAR

    def formula(household, period, parameters):
        # Check if there's a spouse.
        has_spouse = household("is_married", period)
        # Check if there are any dependants.
        has_dependants = add(household, period, ["cwb_dependant"]) > 0
        return has_spouse | has_dependants



from policyengine_canada.model_api import *


class spouse_income(Variable):
    value_type = float
    entity = Person
    label = "Spouse and commonlaw partner income"
    definition_period = YEAR

    def formula(person, period, parameters):
        spouse = person("is_spouse", period)
        income = person("individual_net_income", period)
        return spouse * income



from policyengine_canada.model_api import *


class sk_low_income_credit_base(Variable):
    value_type = float
    entity = Household
    label = "Sasktachewan low income tax credit base"
    definition_period = YEAR
    defined_for = ProvinceCode.SK

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.sk.tax.income.credits.slic
        children = household("sk_low_income_credit_eligible_children", period)
        married = household("is_married", period)
        maximum_amount = p.child.max_number * p.child.amount
        return (
            p.head
            + married * p.spouse
            + min_(children * p.child.amount, maximum_amount)
        )



from policyengine_canada.model_api import *


class is_eldest_child_for_bc_climate_action_tax_credit(Variable):
    value_type = bool
    entity = Person
    label = "Is the first born child in a household, for the purposes of the British Columbia climate action tax credit "
    definition_period = YEAR
    defined_for = "is_child_for_bc_climate_action_tax_credit"

    def formula(person, period, parameters):
        household = person.household
        ix = person("person_index", period)
        child = person("is_child_for_bc_climate_action_tax_credit", period)
        child_index = where(child, ix, 100)
        household_min_child_index = household.min(child_index)
        has_min_child_index = household_min_child_index == ix
        return has_min_child_index & child



from policyengine_canada.model_api import *


class has_private_dental_insurance(Variable):
    value_type = bool
    entity = Person
    label = "Has a private dental insurance plan"
    definition_period = YEAR



from policyengine_canada.model_api import *


class on_child_care_fee_subsidy(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Care Fee Subsidy"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(household, period, parameters):
        reduction = household("on_child_care_fee_subsidy_reduction", period)
        full_time = household("full_time_childcare", period)
        full_time_care = household(
            "on_child_care_fee_subsidy_full_time", period
        )
        part_time_care = household(
            "on_child_care_fee_subsidy_part_time", period
        )
        amount_before_reduction = where(
            full_time, full_time_care, part_time_care
        )
        return max_(0, amount_before_reduction - reduction)



from policyengine_canada.model_api import *


class yt_child_benefit_base(Variable):
    value_type = float
    entity = Household
    label = "Yukon child benefit base"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        children = household("yt_child_benefit_eligible_children", period)
        return (
            children
            * parameters(period).gov.provinces.yt.benefits.child_benefit.base
        )



from policyengine_canada.model_api import *

label = "Earnings"


class employment_income(Variable):
    value_type = float
    entity = Person
    label = "Employment income"
    unit = CAD
    documentation = "Income from gainful employment"
    definition_period = YEAR



from policyengine_core.model_api import *
from policyengine_canada.entities import *
from policyengine_canada.tools.general import *


class ProvinceCode(Enum):
    AB = "AB"
    BC = "BC"
    MB = "MB"
    NB = "NB"
    NL = "NL"
    NS = "NS"
    NT = "NT"
    NU = "NU"
    ONT = "ONT"
    PE = "PE"
    QC = "QC"
    SK = "SK"
    YT = "YT"


class province_code(Variable):
    value_type = Enum
    possible_values = ProvinceCode
    default_value = ProvinceCode.ONT
    entity = Household
    label = "Province code"
    definition_period = ETERNITY

    def formula(household, period, parameters):
        return ProvinceCode.encode(
            household("province_name", period).decode_to_str()
        )



from policyengine_canada.model_api import *


class family_working_income(Variable):
    value_type = float
    entity = Household
    label = "Family working income"
    unit = CAD
    documentation = "Income from employment and self-employment of a family"
    definition_period = YEAR
    # A.2.(122.7)(1.1)
    reference = (
        "https://laws-lois.justice.gc.ca/eng/acts/I-3.3/page-96.html#h-299725"
    )

    adds = ["working_income"]



from policyengine_canada.model_api import *


class dtc_child_supplement(Variable):
    value_type = float
    entity = Person
    label = "Disability child supplement"
    unit = CAD
    definition_period = YEAR
    defined_for = "dtc_eligible"

    def formula(person, period, parameters):
        p = parameters(period).gov.cra.tax.income.credits.dtc
        supplement_eligible = (
            person("age", period) < p.supplement_ineligible_age
        )
        return p.child_supplement * supplement_eligible



from policyengine_canada.model_api import *


class nu_child_benefit_working_component(Variable):
    value_type = float
    entity = Household
    label = "Nunvaut child benefit working component"
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(household, period, parameters):
        children = household("nu_child_benefit_eligible_children", period)
        income = household("family_working_income", period)
        p = parameters(period).gov.provinces.nu.tax.benefits.nucb.working
        phase_in_amount = p.phase_in_rate.calc(income)
        max_amount = p.amount.calc(children)
        return min_(phase_in_amount, max_amount)



from policyengine_canada.model_api import *


class cwb_base_phase_in(Variable):
    value_type = float
    entity = Household
    label = "Canada workers benefit base phase in"
    definition_period = YEAR
    unit = CAD

    def formula(household, period, parameters):
        income = household("family_working_income", period)
        p = parameters(period).gov.cra.benefits.cwb.phase_in.base
        family = household("is_cwb_family", period)
        return where(family, p.family.calc(income), p.single.calc(income))



from policyengine_canada.model_api import *


class oeptc_energy_component(Variable):
    value_type = float
    entity = Household
    label = "OEPTC energy component"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        p = parameters(
            period
        ).gov.provinces.on.tax.income.credits.oeptc.energy_component
        long_term_care_home = (
            household(
                "rent_paid_to_public_or_non_profit_long_term_care_home", period
            )
            * p.multiplier
        )
        reserve_home_energy_costs = household(
            "home_energy_costs_on_a_reserve", period
        )
        student_resident = (
            household("lived_in_a_student_residence", period)
            * p.student_resident_reduction
        )
        occupancy_costs = household("oeptc_occupancy_cost", period)
        uncapped = (
            long_term_care_home
            + reserve_home_energy_costs
            + occupancy_costs
            - student_resident
        )
        return min_(uncapped, p.max_amount)



from policyengine_canada.model_api import *


class on_low_income_workers_tax_credit(Variable):
    value_type = float
    entity = Person
    label = "Ontario Low-Income Workers Tax Credit"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(person, period, parameters):
        household = person.household
        base = person("on_low_income_workers_tax_credit_base", period)
        afni = household("adjusted_family_net_income", period)
        p = parameters(
            period
        ).gov.provinces.on.tax.income.credits.lift.reduction
        family = household("is_married", period)
        reduction = where(family, p.family.calc(afni), p.single.calc(afni))
        return max_(base - reduction, 0)



from policyengine_canada.model_api import *


class ab_spouse_and_common_law_partner_amount_credit(Variable):
    value_type = float
    entity = Household
    label = "Alberta spouse and commonlaw partner amount credit"
    definition_period = YEAR
    defined_for = ProvinceCode.AB

    def formula(household, period, parameters):
        spouse_income = add(household, period, ["spouse_income"])
        p = parameters(
            period
        ).gov.provinces.ab.tax.income.credits.spouse_and_common_law_partner_amount
        return max_(0, (p.base - spouse_income))



from policyengine_canada.model_api import *


class ab_age_credit(Variable):
    value_type = float
    entity = Person
    label = "Alberta age benefit"
    definition_period = YEAR
    defined_for = "ab_age_credit_eligible"

    def formula(person, period, parameters):
        income = person("individual_net_income", period)
        p = parameters(period).gov.provinces.ab.tax.income.credits.age_credit
        return max_(p.base - p.phase_out_rate.calc(income), 0)



from policyengine_canada.model_api import *


class teaching_supplies_expenses(Variable):
    value_type = float
    entity = Person
    label = "Canada expenses on teaching supplies"
    unit = CAD
    documentation = "Amount that was paid for teaching supplies"
    definition_period = YEAR



from policyengine_canada.model_api import *


class qc_taxable_income(Variable):
    value_type = float
    entity = Person
    label = "Quebec taxable income"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.revenuquebec.ca/en/citizens/income-tax-return/completing-your-income-tax-return/income-tax-rates/"
    defined_for = ProvinceCode.QC
    adds = ["total_individual_pre_tax_income"]



from policyengine_canada.model_api import *


class is_dependant(Variable):
    value_type = bool
    entity = Person
    label = "Is a dependant"
    definition_period = YEAR

    # Impute dependant status on age.
    def formula(person, period, parameters):
        return person("age", period) < 18



from policyengine_canada.model_api import *


class pension_and_savings_plan_income(Variable):
    value_type = float
    entity = Person
    label = "Pension and Savings Plan Income"
    unit = CAD
    documentation = "Income from pensions and savings plans"
    definition_period = YEAR



from policyengine_canada.model_api import *


class on_child_care_fee_subsidy_full_time(Variable):
    value_type = float
    entity = Household
    label = "Ontario Child Care Fee Subsidy in full time care"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(
            period
        ).gov.provinces.on.subsidies.on_child_care_fee_subsidy
        return p.full_time_calculation.calc(income)



from policyengine_canada.model_api import *


class disability_tax_credit(Variable):
    value_type = float
    entity = Person
    label = "Disability tax credit"
    unit = CAD
    definition_period = YEAR
    adds = ["dtc_base", "dtc_child_supplement"]



from policyengine_canada.model_api import *


class lived_in_a_student_residence(Variable):
    value_type = bool
    entity = Household
    label = "Student residence"
    unit = CAD
    documentation = "A person who lived in a stundet residence"
    definition_period = YEAR



from policyengine_canada.model_api import *


class mb_basic_personal_amount(Variable):
    value_type = float
    entity = Person
    label = "Manitoba basic personal amount"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.canada.ca/content/dam/cra-arc/formspubs/pbg/td1mb/td1mb-23e.pdf#page=1"
    defined_for = ProvinceCode.MB

    def formula(person, period, parameters):
        p = parameters(period).gov.provinces.mb.credits
        return p.basic_personal_amount



from policyengine_canada.model_api import *


class age_amount_credit(Variable):
    value_type = float
    entity = Person
    label = "Age amount credit, non-refundable"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(period).gov.cra.tax.income.credits.age_amount_credit
        eligible = age >= p.age_eligibility
        income = person("total_individual_pre_tax_income", period)
        credit_cap = p.amount.credit_cap
        reduction = p.amount.reduction.calc(income)
        return eligible * (max_(0, credit_cap - reduction))



from policyengine_canada.model_api import *


class on_low_income_workers_tax_credit_base(Variable):
    value_type = float
    entity = Person
    label = "Base amount of Ontario Low-Income Workers Tax Credit"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        household = person.household
        employment_income = person("employment_income", period)
        p = parameters(period).gov.provinces.on.tax.income.credits.lift
        eligible_people = person(
            "on_low_income_workers_tax_credit_eligible_people", period
        )
        return min_(
            eligible_people * p.amount,
            employment_income * p.phase_in_rate,
        )



from policyengine_canada.model_api import *


class household_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = Household
    label = "tax"
    documentation = "Total tax liability before refundable credits."
    unit = CAD
    definition_period = YEAR
    adds = [
        "income_tax_before_refundable_credits",  # Federal.
        "on_income_tax_before_refundable_credits",  # Ontario.
        "bc_income_tax_before_refundable_credits",  # British Columbia.
    ]



from policyengine_canada.model_api import *


class nu_child_benefit_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Nunvaut child benefit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(person, period, parameters):
        age = person("age", period)
        return (
            age
            < parameters(period).gov.provinces.nu.tax.benefits.nucb.age_limit
        )



from policyengine_canada.model_api import *


class qc_sa_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "Quebec senior assistance tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.qc.tax.income.credits.sa

        income = household("adjusted_family_net_income", period)

        # Your spouse is an eligible individual and you were both 70 or over
        married_both_eligible = household(
            "qc_sa_married_both_eligible", period
        )
        credit_married_both_eligible = married_both_eligible * max_(
            0,
            p.amount.married_both_eligible - p.reduction.married.calc(income),
        )

        # Your spouse is not an eligible individual or only one of you was 70 or over
        married_one_eligible = household("qc_sa_married_one_eligible", period)
        credit_married_one_eligible = married_one_eligible * max_(
            0, p.amount.married_one_eligible - p.reduction.married.calc(income)
        )

        # You did not have a spouse
        single = household("qc_sa_single", period)
        credit_single = single * max_(
            0, p.amount.single - p.reduction.single.calc(income)
        )

        return (
            credit_married_both_eligible
            + credit_married_one_eligible
            + credit_single
        )



from policyengine_canada.model_api import *


class ns_income_assistance_age_eligibility(Variable):
    value_type = bool
    entity = Person
    label = "Nova Scotia income assistance age eligibility"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(
            period
        ).gov.provinces.ns.tax.income.income_assistance.eligibility.age
        protective_services = person("in_need_of_protective_services", period)
        # Person is eligible if at or over 19 years of age
        # Person is eligible if between 16 and 19 years of age,
        # and is in need of protective services
        # The person in need of protective services is additionally required to:
        # attend an educational program not designated for student loan purposes;
        # participate in an employment plan;
        # access counselling or mediation services that was identified as necessary;
        # access medical services necessary to preserve their physical health;
        # live in a setting that provides a degree of supervision, accountability,
        # and guidance in accordance with their age and needs.
        # which are not modeled in the program.
        age_threshold = where(protective_services, p.lower, p.main)
        return age >= age_threshold



from policyengine_canada.model_api import *


class school_supply_credit(Variable):
    value_type = float
    entity = Person
    label = "School supply credit"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        expenses = person("teaching_supplies_expenses", period)
        p = parameters(period).gov.cra.tax.income.credits.school_supply_credit
        return min_(expenses * p.amount, p.cap)



from policyengine_canada.model_api import *


class child_disability_benefit(Variable):
    value_type = float
    entity = Household
    label = "Child disability benefit"
    unit = CAD
    documentation = "Non taxable amount paid monthly per children with a severe and prolonged impairment in physical or mental functions."
    definition_period = YEAR

    def formula(household, period, parameters):
        reduction = household("child_disability_benefit_reduction", period)
        base = parameters(period).gov.cra.benefits.cdb.base
        children = household("child_disability_benefit_children", period)
        return max_(base * children - reduction, 0)



from policyengine_canada.model_api import *


class family_net_income(Variable):
    value_type = float
    entity = Household
    label = "Family net income"
    unit = CAD
    documentation = "Sum of head and spouse's line 23600 on tax return"
    definition_period = YEAR
    adds = ["individual_net_income"]



from policyengine_canada.model_api import *


class is_child_for_bc_climate_action_tax_credit(Variable):
    value_type = bool
    entity = Person
    label = "Is the first born child in a Household"
    definition_period = YEAR
    defined_for = "is_dependant"

    def formula(person, period, parameters):
        adult_age = parameters(
            period
        ).gov.provinces.bc.tax.income.credits.bccatc.adult_age
        return person("age", period) < adult_age
        # The definition of an eligible child also includes living with
        # parents and eligibilty under CCB. These are not included here.



from policyengine_canada.model_api import *


class yt_childrens_arts_credit_eligible_child(Variable):
    value_type = bool
    entity = Person
    label = "Yukon childrens arts credit eligible child"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(person, period, parameters):
        age = person("age", period)
        return (
            age
            < parameters(
                period
            ).gov.provinces.yt.tax.income.credits.childrens_arts_credit.child_age_eligibility
        )



from policyengine_canada.model_api import *


class gst_credit_person(Variable):
    value_type = float
    entity = Person
    label = "GST credit amount for a particular individual in a household. We'll sum these in a separate variable."
    unit = CAD
    documentation = "Determination of the amount per individual"
    definition_period = YEAR

    def formula(person, period, parameters):
        category = person("gst_credit_category", period)
        amounts = parameters(
            period
        ).gov.cra.tax.income.credits.gst_credit.base_amounts
        return amounts[category]



from policyengine_canada.model_api import *


class nu_benefits(Variable):
    value_type = float
    entity = Household
    label = "Nunavut benefits"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.NU
    adds = "gov.provinces.nu.benefits.benefits"



from policyengine_canada.model_api import *


class on_sales_tax_credit_reduction(Variable):
    value_type = float
    entity = Household
    label = "Ontario Sales Tax Credit reduction"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        family = household("household_size", period) > 1
        p = parameters(
            period
        ).gov.provinces.on.tax.income.credits.ostc.reduction
        return where(family, p.family.calc(income), p.single.calc(income))



from policyengine_canada.model_api import *


class bc_family_benefit_eligible_children(Variable):
    value_type = float
    entity = Household
    label = "British Columbia family benefit eligible children"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC
    adds = ["bc_family_benefit_eligible_child"]



from policyengine_canada.model_api import *


class on_per_child_care_subsidy_costs(Variable):
    value_type = float
    entity = Household
    label = "Ontario per child care subsidy costs"
    unit = CAD
    definition_period = YEAR



from policyengine_canada.model_api import *


class in_whitehorse(Variable):
    value_type = bool
    entity = Household
    label = "Household that resides in Whitehorse"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.YT



from policyengine_canada.model_api import *


class child_disability_benefit_reduction(Variable):
    value_type = float
    entity = Household
    label = "Child Disability Benefit reduction"
    unit = CAD
    definition_period = YEAR

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        children = household("child_disability_benefit_children", period)
        p = parameters(period).gov.cra.benefits.cdb.reduction
        return select(
            # Conditions.
            [children == 1, children > 1],
            # Results.
            [p.one_child.calc(income), p.two_or_more_children.calc(income)],
            default=0,
        )



from policyengine_canada.model_api import *


class bc_climate_action_tax_credit(Variable):
    value_type = float
    entity = Household
    label = "British Columbia climate action tax credit"
    unit = CAD
    documentation = "Universal amount without adjustment based on AFNI"
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        married = household("is_married", period)
        single_parent = household(
            "bc_climate_action_tax_credit_single_parent_household", period
        )
        family = married | single_parent
        base = household("bc_climate_action_tax_credit_base", period)
        p = parameters(period).gov.provinces.bc.tax.income.credits.bccatc
        reduction = where(
            family,
            p.reduction_rate.family.calc(income),
            p.reduction_rate.single.calc(income),
        )
        return max_(base - reduction, 0)



from policyengine_canada.model_api import *


class nu_child_benefit_base_component_reduction(Variable):
    value_type = float
    entity = Household
    label = "Nunvaut child benefit base component reduction"
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    def formula(household, period, parameters):
        income = household("adjusted_family_net_income", period)
        p = parameters(period).gov.provinces.nu.tax.benefits.nucb.base
        return p.phase_out_rate.calc(income)



from policyengine_canada.model_api import *


class count_children(Variable):
    value_type = int
    entity = Household
    label = "Children"
    unit = CAD
    documentation = "Number of dependant children under the age of 19"
    definition_period = YEAR



from policyengine_canada.model_api import *


class sk_senior_supplementary_credit(Variable):
    value_type = float
    unit = CAD
    entity = Person
    label = "Sasktachewan senior supplementary tax credit"
    definition_period = YEAR
    defined_for = "sk_senior_supplementary_credit_eligible"

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.provinces.sk.tax.income.credits.sk_senior_supplementary
        eligibility = person("sk_senior_supplementary_credit_eligible", period)
        return eligibility * p.amount



from policyengine_canada.model_api import *


class qc_income_tax_before_credits(Variable):
    value_type = float
    entity = Person
    label = "Quebec income tax"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.revenuquebec.ca/en/citizens/income-tax-return/completing-your-income-tax-return/income-tax-rates/"
    defined_for = ProvinceCode.QC

    def formula(person, period, parameters):
        income = person("qc_taxable_income", period)
        p = parameters(period).gov.provinces.qc.tax.income.rate
        return p.calc(income)



from policyengine_canada.model_api import *


class qc_cost_of_living_credit(Variable):
    value_type = float
    entity = Person
    label = "Quebec cost of living credit"
    definition_period = YEAR
    defined_for = ProvinceCode.QC

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.provinces.qc.tax.income.credits.cost_of_living
        income = person("individual_net_income", period)

        # determine eligibility:
        # You were 18 or over or, if you were under 18, you:
        # (a) were the father or mother of a child who lived with you; or
        # (b) were recognized as an emancipated minor by a competent authority (such as a court)
        is_adult = person("is_adult", period)

        # (a)
        is_parent = person("own_children_in_household", period) > 0
        # (b)
        is_emancipated = person("is_emancipated", period)

        eligible = is_adult | is_parent | is_emancipated
        amount_if_eligible = max_(0, p.base - p.reduction.calc(income))
        return eligible * amount_if_eligible



from policyengine_canada.model_api import *


class ns_low_income_tax_reduction_base(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia low income tax reduction base"
    definition_period = YEAR
    defined_for = ProvinceCode.NS

    def formula(household, period, parameters):
        person = household.members
        p = parameters(
            period
        ).gov.provinces.ns.tax.income.credits.low_income_tax_reduction
        age = person("age", period)
        eligible = age > p.age_eligibility
        base = p.base.base
        spouse = person("is_spouse", period)
        spouse_amount = spouse * p.base.spouse
        dependant = person("is_dependant", period)
        dependant_amount = dependant * p.base.eligible_dependant
        return min_(
            p.base.max_amount,
            household.sum(
                eligible * (base + spouse_amount + dependant_amount)
            ),
        )



from policyengine_canada.model_api import *


class investment_income(Variable):
    value_type = float
    entity = Person
    label = "Investment income"
    unit = CAD
    documentation = (
        "Income from investments including dividends and capital gains"
    )
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/tax/individuals/topics/about-your-tax-return/tax-return/completing-a-tax-return/personal-income/self-employment-income-lines-13499-14299-gross-income-lines-13500-14300-net-income.html"



from policyengine_canada.model_api import *


class bc_climate_action_tax_credit_single_parent_household(Variable):
    value_type = bool
    entity = Household
    label = "British Columbia Climate Action single parent status"
    unit = CAD
    documentation = "Determination wether the filer is a single parent eligible for the climate action tax credit"
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(household, period, parameters):
        married = household("is_married", period)
        bccatc_children = household(
            "bc_climate_action_tax_credit_dependant_children", period
        )
        return ~married & (bccatc_children > 0)



from policyengine_canada.model_api import *


class yt_government_carbon_price_rebate(Variable):
    value_type = float
    entity = Household
    label = "Yukon government carbon price rebate"
    unit = CAD
    definition_period = YEAR
    reference = "https://www.canada.ca/en/revenue-agency/services/child-family-benefits/provincial-territorial-programs/yukon.html#YGCPRI"
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        p = parameters(period).gov.provinces.yt.benefits.rebates.ygcpri
        person = household.members
        child = person("age", period) < p.child_ineligible_age
        children = household.sum(child)
        spouse = person("is_spouse", period)
        spouses = household.sum(spouse)
        in_whitehorse = household("in_whitehorse", period)
        non_whitehorse_supplement = ~in_whitehorse * (
            (p.non_whitehorse_supplement.child * children)
            + (p.non_whitehorse_supplement.spouse * spouses)
            + p.non_whitehorse_supplement.self
        )
        base = (
            (p.amount.child * children)
            + (p.amount.spouse * spouses)
            + p.amount.self
        )
        return base + non_whitehorse_supplement



from policyengine_canada.model_api import *


class ns_refundable_credits(Variable):
    value_type = float
    entity = Household
    label = "Nova Scotia refundable tax credits"
    unit = CAD
    definition_period = YEAR

    adds = "gov.provinces.ns.tax.income.credits.refundable"



from policyengine_canada.model_api import *


class bc_climate_action_tax_credit_base(Variable):
    value_type = float
    entity = Household
    label = "British Columbia Climate Action tax Credit base amount"
    unit = CAD
    documentation = "Universal amount without adjustment based on AFNI"
    definition_period = YEAR
    defined_for = ProvinceCode.BC
    adds = ["bc_climate_action_tax_credit_person"]



from policyengine_canada.model_api import *


class oas_repayment(Variable):
    value_type = float
    entity = Person
    label = "Old age security pension repayment"
    documentation = "Your repayment calculation is based on the difference between your income and the threshold amount for the year. The first step is to figure out how much higher your income is than the threshold. You must repay 15 percent of that amount. See OASTD and OASRR in the SPSD/M Parameter Guide"
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        p = parameters(period).gov.cra.benefits.old_age_security_pension
        # The income concept in the documentation is 'net world income'. I just use net income here.
        net_income = person("individual_net_income", period)
        # The repayment tax is a percent of income above a threshold.
        uncapped_repayment_tax = p.repayment_tax.calc(net_income)
        # Cap repayment at the pre-repayment amount.
        oas_pre_repayment = person("oas_pre_repayment", period)
        repayment_tax = min_(oas_pre_repayment, uncapped_repayment_tax)

        # Round to the nearest cent.
        return numpy.around(repayment_tax, 2)



from policyengine_canada.model_api import *


class full_custody(Variable):
    value_type = bool
    entity = Person
    label = "Is in full custody of the household head"
    definition_period = YEAR



from policyengine_canada.model_api import *


class yt_childrens_arts_credit(Variable):
    value_type = float
    entity = Household
    label = "Yukon childrens arts credit"
    definition_period = YEAR
    defined_for = ProvinceCode.YT

    def formula(household, period, parameters):
        children = household(
            "yt_childrens_arts_credit_eligible_children", period
        )
        expenses = household("yt_childrens_arts_credit_expenses", period)
        p = parameters(
            period
        ).gov.provinces.yt.tax.income.credits.childrens_arts_credit
        return min_(expenses, children * p.amount)



from policyengine_canada.model_api import *


class bc_family_benefit_eligible_child(Variable):
    value_type = float
    entity = Person
    label = "British Columbia family benefit eligible child"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.BC

    def formula(person, period, parameters):
        age = person("age", period)
        p = parameters(period).gov.provinces.bc.benefits.bcfb
        return age < p.child_ineligible_age_threshold



from policyengine_canada.model_api import *


class nu_child_benefit(Variable):
    value_type = float
    entity = Household
    label = "Nunvaut child benefit"
    definition_period = YEAR
    defined_for = ProvinceCode.NU

    adds = [
        "nu_child_benefit_base_component",
        "nu_child_benefit_working_component",
    ]



from policyengine_canada.model_api import *


class dental_benefit(Variable):
    value_type = float
    entity = Person
    label = "Canada dental benefit"
    unit = CAD
    definition_period = YEAR
    defined_for = "dental_benefit_eligible"

    def formula(person, period, parameters):
        income = person.household("adjusted_family_net_income", period)
        p = parameters(period).gov.cra.benefits.dental_benefit
        full_custody_amount = p.amount.calc(income)
        # Multiply by a factor for shared custody.
        return full_custody_amount * where(
            person("full_custody", period), 1, p.shared_custody_share
        )



from policyengine_canada.model_api import *

# SPSD/M 29.0: imioas or imoasmax, I'm not sure. I think imioas is post-repayment?


class oas_pre_repayment(Variable):
    value_type = float
    entity = Person
    label = "Old age security pension pre-repayment"
    documentation = "The OAS amount a person is eligible for prior to the repayment tax. See SPSD/M 'imoasmax'."
    unit = CAD
    definition_period = YEAR

    def formula(person, period, parameters):
        age = person("age", period)
        adult_years_in_canada = person("adult_years_in_canada", period)
        oas_eligible = person("oas_eligibility", period)
        p = parameters(period).gov.cra.benefits.old_age_security_pension
        # Age at which you get the percentage boost.
        older_increase_age_threshold = p.eligibility.age.older_seniors_increase
        eligible_older_increase = age >= older_increase_age_threshold
        total_older_increase_factor = (
            1
            + eligible_older_increase * p.amount.older_seniors_increase_factor
        )
        base_amount = p.amount.base
        # Your full base amount is your number of adult residence years divided
        # by the number of years at which you are eligible for 100%.
        # In the SPSD/M 29.0 this is imoasres.
        residency_scale_factor = min_(
            adult_years_in_canada / p.eligibility.residence.full_base_amount,
            1,
        )
        # Apply the old age boost to the base amount if applicable.
        # In the SPSD/M 29.0 this is imoasmax.
        amount = (
            oas_eligible
            * base_amount
            * residency_scale_factor
            * total_older_increase_factor
        )

        return numpy.around(amount, 2)



from policyengine_canada.model_api import *


class on_senior_homeowners_property_tax_grant(Variable):
    value_type = float
    entity = Person
    label = "Ontario senior homeowners property tax grant"
    unit = CAD
    definition_period = YEAR
    defined_for = ProvinceCode.ONT

    def formula(person, period, parameters):
        base = person("on_senior_homeowners_property_tax_grant_base", period)
        reduction = person(
            "on_senior_homeowners_property_tax_grant_reduction", period
        )
        return max_(0, base - reduction)
